Type: Package
Package: corrgram
Title: Plot a Correlogram
Version: 1.15
Date: 2026-05-18
Authors@R: 
    person("Kevin", "Wright", , "kw.stat@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0002-0617-8673"))
Description: Calculates correlation of variables and displays the results
    graphically. Included panel functions can display points, shading,
    ellipses, and correlation values with confidence intervals. See
    Friendly (2002) <doi:10.1198/000313002533>.
License: MIT + file LICENSE
URL: https://kwstat.github.io/corrgram/
BugReports: https://github.com/kwstat/corrgram/issues/
Imports: ellipse, graphics, grDevices, grid, lattice, stats
Suggests: gridBase, hexbin, knitr, MASS, Matrix, psych, rmarkdown,
        seriation, sfsmisc, testthat
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-18 17:29:13 UTC; wrightkevi
Author: Kevin Wright [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-0617-8673>)
Maintainer: Kevin Wright <kw.stat@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-18 19:10:02 UTC
Built: R 4.6.0; ; 2026-05-18 20:09:56 UTC; unix

# Generated by roxygen2: do not edit by hand

export(corrgram)
export(corrgram2)
export(grid_panel.conf)
export(grid_panel.ellipse)
export(grid_panel.fill)
export(grid_panel.pie)
export(grid_panel.pts)
export(grid_panel.shade)
export(grid_text.panel)
export(levelplot_panel.ellipse)
export(levelplot_panel.pie)
export(panel.bar)
export(panel.conf)
export(panel.cor)
export(panel.density)
export(panel.ellipse)
export(panel.fill)
export(panel.minmax)
export(panel.pie)
export(panel.pts)
export(panel.shade)
export(panel.txt)
export(splom_panel.ellipse)
export(splom_panel.pie)
export(splom_panel.shade)
import(grDevices)
import(graphics)
import(grid)
import(stats)
importFrom(ellipse,ellipse)
importFrom(lattice,current.panel.limits)
importFrom(lattice,level.colors)
importFrom(lattice,levelplot)
importFrom(lattice,llines)
importFrom(lattice,panel.polygon)
importFrom(lattice,panel.rect)
importFrom(lattice,panel.text)
importFrom(lattice,panel.xyplot)
importFrom(lattice,splom)
importFrom(utils,installed.packages)

## ----setup, include=FALSE---------------------------------------------------------------
library(corrgram)

## ----vote-example, fig.width=6, fig.height=6--------------------------------------------
corrgram2(vote, abs = TRUE, order = TRUE, legend = TRUE, title = "vote data")

## ----auto-example, fig.width=6, fig.height=6--------------------------------------------
vars6 <- setdiff(colnames(auto), c("Model", "Origin"))
corrgram2(auto[, vars6], 
          lower.panel = grid_panel.shade, upper.panel=grid_panel.pie,
          title = "auto data", legend = TRUE)

## ----corrgram-lattice-correlation-matrix, fig.width=6, fig.height=6---------------------
library("lattice")
library("corrgram")

# The easiest way to have an automatic color key is to set the theme
opar <- trellis.par.get()
trellis.par.set(
  regions=list(col=colorRampPalette(c("red","salmon","white","royalblue","navy")))
)

# Create a correlation matrix
library(MASS) # foor Cars93
cor.Cars93 <-   cor(Cars93[, !sapply(Cars93, is.factor)], use = "pair")
ord <- order.dendrogram(as.dendrogram(hclust(dist(cor.Cars93))))
cars93 <- cor.Cars93[ord,ord]
head(cars93)

# lattice corrgram using pie-shaped glyphs
levelplot(cars93, xlab = NULL, ylab = NULL,
          at = do.breaks(c(-1.01, 1.01), 101), panel = levelplot_panel.pie,
          scales = list(x = list(rot = 90)), colorkey = list(space = "top") )

# lattice corrgram using ellipse-shaped glyphs above the diagonal
# and value labels below the diagonal
levelplot(cars93, xlab = NULL, ylab = NULL,
          at = do.breaks(c(-1.01, 1.01), 101), panel = levelplot_panel.ellipse,
          label=TRUE,
          scales = list(x = list(rot = 90)), colorkey = list(space = "top") )


## ---------------------------------------------------------------------------------------
pengvars <- c("bill_len", "bill_dep", "flipper_len", "body_mass")
library(lattice)
splom(~penguins[ , pengvars], upper.panel=splom_panel.pie, pscales=0)
splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.pie, pscales=0)
splom(~penguins[ , pengvars], upper.panel=splom_panel.shade, pscales=0)
splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.shade, pscales=0)
splom(~penguins[ , pengvars], upper.panel=splom_panel.ellipse, pscales=0)
splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.ellipse, pscales=0)

## ---------------------------------------------------------------------------------------
# Hexbin
library(lattice)
library(hexbin)
splom(~penguins[ , pengvars], upper.panel=hexbin::panel.hexbinplot, pscales=0)
splom(~penguins[ , pengvars]|penguins$species, upper.panel=hexbin::panel.hexbinplot, pscales=0)

## ----setup, results="hide"--------------------------------------------------------------
library("knitr")
opts_chunk$set(fig.align="center", fig.width=6, fig.height=6)
options(width=90)

## ----data-------------------------------------------------------------------------------
library("corrgram")
head(baseball)
round(cor(baseball[, 5:14], use="pair"),2)

## ----fig2-------------------------------------------------------------------------------
vars2 <- c("Assists","Atbat","Errors","Hits","Homer","logSal",
           "Putouts","RBI","Runs","Walks","Years")
corrgram(baseball[,vars2], order=TRUE,
         main="Baseball data PC2/PC1 order",
         lower.panel=panel.shade, upper.panel=panel.pie,
         diag.panel=panel.minmax, text.panel=panel.txt)

## ----fig3-------------------------------------------------------------------------------
baseball.cor <- cor(baseball[,vars2], use='pair')
baseball.eig <- eigen(baseball.cor)$vectors[,1:2]
e1 <- baseball.eig[,1]
e2 <- baseball.eig[,2]
plot(e1,e2,col='white', xlim=range(e1,e2), ylim=range(e1,e2))
text(e1,e2, rownames(baseball.cor), cex=1)
title("Eigenvector plot of baseball data")
arrows(0, 0, e1, e2, cex=0.5, col="red", length=0.1)

## ----fig4-------------------------------------------------------------------------------
corrgram(baseball[,vars2], main="Baseball data (alphabetic order)")

corrgram(baseball[,vars2], order=TRUE,
         main="Baseball data (PC order)",
         panel=panel.shade, text.panel=panel.txt)

## ----fig5-------------------------------------------------------------------------------
corrgram(baseball, order=TRUE, main="Baseball data (PC order)")

## ----fig6-------------------------------------------------------------------------------
corrgram(auto, order=TRUE, main="Auto data (PC order)")

## ----fig7-------------------------------------------------------------------------------
rinv <- function(r){
  # r is a correlation matrix
  # calculate r inverse and scale to correlation matrix
  # Derived from Michael Friendly's SAS code

  ri <- solve(r)
  s <- diag(ri)
  s <- diag(sqrt(1/s))
  ri <- s %*% ri %*% s
  n <- nrow(ri)
  ri <- ri * (2*rep(1,n) - matrix(1, n, n))
  diag(ri) <- 1  # Should already be 1, but could be 1 + epsilon
  colnames(ri) <- rownames(ri) <- rownames(r)
  return(ri)
}

vars7 <- c("Years", "logSal", "Homer", "Putouts", "RBI", "Walks",
           "Runs", "Hits", "Atbat", "Errors", "Assists")
cb <- cor(baseball[,vars7], use="pair")
corrgram(-rinv(cb), main=expression(paste("Baseball data ", R^-1)))


## ----fig8-------------------------------------------------------------------------------
require(Matrix) # For block diagonal function

partial <- function(r, xvar){
  # r is a correlation matrix
  # Calculate partial correlation of y|x
  yvar <- setdiff(colnames(r), xvar)
  ri <- r[yvar,yvar] - r[yvar,xvar] %*% solve(r[xvar,xvar]) %*% r[xvar,yvar]
  s <- diag(ri)
  s <- diag(sqrt(1/s))
  ri <- s %*% ri %*% s
  ri <- as.matrix(Matrix::bdiag(ri, r[xvar, xvar]))
  diag(ri) <- 1  # Should already be 1, but could be 1 + epsilon
  colnames(ri) <- rownames(ri) <- c(yvar, xvar)
  return(ri)
}

vars8a <- c("Gratio", "Rep78", "Rep77", "Hroom", "Trunk", "Rseat",
            "Length", "Weight", "Displa", "Turn")
vars8b <- c("MPG", "Price")
vars8 <- c(vars8a, vars8b)
auto.cor <- cor(auto[, vars8], use="pair")
auto.par <- partial(auto.cor, vars8b)
corrgram(auto.par,
         lower.panel=panel.pie, upper.panel=panel.pie,
         main="Auto data, partialing out Price,MPG")

## ----ellipse----------------------------------------------------------------------------
corrgram(baseball[,vars2], order=TRUE,
         main="Baseball correlation ellipses",
         panel=panel.ellipse,
         text.panel=panel.txt, diag.panel=panel.minmax)

## ----scatter----------------------------------------------------------------------------
corrgram(iris,
         main="Iris data with example panel functions",
         lower.panel=panel.pts, upper.panel=panel.conf,
         diag.panel=panel.density)

## ----panelbar---------------------------------------------------------------------------
corrgram(auto, order=TRUE,
         main="Auto data (PC order)",
         lower.panel=corrgram::panel.ellipse,
         upper.panel=panel.bar, diag.panel=panel.minmax,
         col.regions=colorRampPalette(c("darkgoldenrod4", "burlywood1",
                                        "darkkhaki", "darkgreen")))

## ----vote-------------------------------------------------------------------------------
# 'vote' is a correlation matrix, not a data frame
corrgram(vote, order=TRUE,
         upper.panel=panel.cor, main="vote")

## ---------------------------------------------------------------------------------------

load(url("https://github.com/alexanderrobitzsch/sirt/blob/master/data/data.ratings3.rda?raw=true"))

# jitter first, so the upper/lower panels are symmetric
data.ratings3 <- transform(data.ratings3,
                           c2=jitter(crit2), c3=jitter(crit3),
                           c4=jitter(crit4), c6=jitter(crit6))
                           
library(corrgram)
panel.raters <- function (x, y, corr = NULL, col.regions, cor.method, ...) {
  if (!is.null(corr)) 
    return()
  plot.xy(xy.coords(x, y), type = "p", ...)
  abline(lm(y ~ x))
  box(col = "lightgray")
}
corrgram(data.ratings3[,7:10], diag=panel.density, lower.panel=panel.raters, upper.panel=panel.conf)

# corrgram_splom.R

#' Panel functions for lattice corrgrams via splom
#'
#' These functions provide custom panel methods for \code{lattice::splom()}.
#'
#' @param x Numeric coordinates from levelplot.
#' @param y Numeric coordinates from levelplot.
#' @param z Correlation values from levelplot.
#' @param subscripts Subscripts for lattice panel. (not used)
#' @param at Breaks for color levels.
#' @param cor.method Correlation method (default "pearson").
#' @param ... Additional arguments passed to panel functions.
#'
#' @details
#' 
#' \code{splom_panel.pie} Draws pie glyphs representing correlation coefficients,
#' omitting the diagonal.
#' 
#' \code{splom_panel.shade} Draws shaded rectangles with hash lines representing correlation
#' coefficients.
#' 
#' \code{splom_panel.ellipse} Draws ellipses representing correlation coefficients.
#' The position of the ellipse is determined by the position of the data,
#' and the shape of the ellipse is determined by the correlation.
#' 
#' @rdname splom_panel.pie
#' @importFrom ellipse ellipse
#' @importFrom lattice level.colors panel.polygon panel.text
#' @examples
#' library(lattice)
#' pengvars <- c("bill_len", "bill_dep", "flipper_len", "body_mass")
#' splom(~penguins[ , pengvars], upper.panel=splom_panel.pie, pscales=0)
#' splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.pie, pscales=0)
#' splom(~penguins[ , pengvars], upper.panel=splom_panel.shade, pscales=0)
#' splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.shade, pscales=0)
#' splom(~penguins[ , pengvars], upper.panel=splom_panel.ellipse, pscales=0)
#' splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.ellipse, pscales=0)
#' @importFrom lattice current.panel.limits panel.xyplot panel.polygon panel.rect splom
#' @export
splom_panel.pie <- function(x, y, z, subscripts, at = pretty(z), 
                      cor.method="pearson", ...) {
  # Each glyph is in a single panel
  # x and y are for only this panel...do not subset by subscripts

  # Corner coordinates of panel box, which is NOT square
  cpl <- lattice::current.panel.limits()
  xmin <- cpl$xlim[1]
  xmax <- cpl$xlim[2]
  ymin <- cpl$ylim[1]
  ymax <- cpl$ylim[2]

  # Multiply the radius by .97 so the circles do not overlap
  rx <- (xmax-xmin)/2 * .97
  ry <- (ymax-ymin)/2 * .97
  centerx <- (xmin+xmax)/2
  centery <- (ymin+ymax)/2

  # Don't think z is passed in as an argument
  if(missing(z)) {
    z <- cor(x,y, use="pairwise.complete.obs", method=cor.method)
  } else {
    z <- as.numeric(z)
  }
  zcol <- level.colors(z, at = at, ...)
  
  # Draw circle
  segments <- 60
  angles <- seq(0,2*pi,length=segments)
  circ <- cbind(centerx + cos(angles)*rx, centery + sin(angles)*ry)
  panel.xyplot(circ[,1], circ[,2], type = "l",
               lty = 1, col="gray30", ...)

  # Overlay a colored pacman polygon
  n_bins <- 14
  pal <- colorRampPalette(c("red","salmon","white","royalblue","navy"))(n_bins)
  col.ind <- as.numeric(cut(z, breaks=seq(from=-1, to=1, length=n_bins+1),
                            include.lowest=TRUE))
  col.pie <- pal[col.ind]
  segments <- round(60*abs(z),0)
  if(segments>0){ # Watch out for the case with 0 segments
    angles <- seq(pi/2, pi/2+(2*pi* -z), length=segments)
    circ <- cbind(centerx + cos(angles)*rx, centery + sin(angles)*ry)
    circ <- rbind(circ, c(centerx, centery), circ[1,])
    panel.polygon(circ[,1], circ[,2], col=col.pie)
  }
}
#splom(~penguins[ , pengvars], upper.panel=splom_panel.pie, pscales=0)
#splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.pie, pscales=0)

# -----------------------------------------------------------------------------

#' @rdname splom_panel.pie
#' @param col.regions Color palette for shading (default NULL uses internal
#' red white blue palette).
#' @export
splom_panel.shade <- function(x, y, z, subscripts, at = pretty(z),
  col.regions = NULL,
  cor.method = "pearson", ...) {
  # x and y are already subsetted for this panel
  if (length(x) < 2 || length(y) < 2 || 
    all(is.na(x)) || all(is.na(y))) return()

  # Compute correlation
  # Don't think z is passed in as an argument
  if(missing(z)) {
    z <- cor(x,y, use="pairwise.complete.obs", method=cor.method)
  } else {
    z <- as.numeric(z)
  }
  zcol <- level.colors(z, at = at, ...)


  # Color palette
  if (is.null(col.regions)) {
    pal <- colorRampPalette(c("red","salmon","white","royalblue","navy"))(14)
  } else {
    pal <- col.regions(14)
  }
  col.ind <- as.numeric(cut(z, breaks = seq(-1, 1, length.out = 15), include.lowest = TRUE))
  col.shade <- pal[col.ind]

  # Panel limits
  cpl <- lattice::current.panel.limits()
  xmin <- cpl$xlim[1]
  xmax <- cpl$xlim[2]
  ymin <- cpl$ylim[1]
  ymax <- cpl$ylim[2]

  # Solid fill
  panel.rect(xmin, ymin, xmax, ymax, col = col.shade)

  # Hash lines using [0,1] coordinates for the panel
  if(z >0) {
    # 5 lines with slope 1
    x0 = c(0, 0, 0, .33, .67)
    x1 = c(.33, .67, 1, 1, 1)
    y0 = c(.67, .33, 0, 0, 0)
    y1 = c(1, 1, 1, .67, .33)
  } else {
    # slope -1
    x0 = c(0, 0, 0, .33, .67)
    x1 = c(.33, .67, 1, 1, 1)
    y0 = c(.33, .67, 1, 1, 1)
    y1 = c(0, 0, 0, .33, .67)
  }
  
  grid::grid.segments(
      x0 = x0, y0 = y0,
      x1 = x1, y1 = y1,
      default.units = "npc",
      gp = grid::gpar(col = "white", lwd = 1, lty = 1, ...)
    )

}

#splom(~penguins[ , pengvars], upper.panel=splom_panel.shade, pscales=0)
#splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.shade, pscales=0)

# ----------------------------------------------------------------------------

#' @rdname splom_panel.pie
#' @importFrom lattice llines panel.xyplot panel.rect
#' @export
splom_panel.ellipse <- function(x,y, col.regions, cor.method="pearson", ...){

  # For correlation matrix, do nothing
  #if(!is.null(corr)) return()

  # If too few points in the panel, do nothing
  if(sum(complete.cases(x,y)) < 2) {
    warning("Need at least 2 complete cases to draw ellipse.")
    return()
  }

  # Draw an unfilled ellipse
  dfn <- 2
  dfd <- length(x)-1
  shape <- var(cbind(x,y),na.rm=TRUE)
  keep <- (!is.na(x) & !is.na(y))
  center <- c(mean(x[keep]),mean(y[keep]))
  radius <- sqrt(dfn*qf(.68,dfn,dfd))
  segments <- 180
  angles <- seq(from=0, to=2*pi, length.out=segments)
  unit.circle <- cbind(cos(angles),sin(angles))
  ellipse.pts <- t(center+radius*t(unit.circle%*%chol(shape)))
  ellx <- ellipse.pts[,1]
  elly <- ellipse.pts[,2]

  # Truncate ellipse at min/max or at bounding box
  cpl <- lattice::current.panel.limits()
  xmin <- cpl$xlim[1]
  xmax <- cpl$xlim[2]
  ymin <- cpl$ylim[1]
  ymax <- cpl$ylim[2]
  ellx <- ifelse(ellx < xmin, xmin, ellx)
  ellx <- ifelse(ellx > xmax, xmax, ellx)
  elly <- ifelse(elly < ymin, ymin, elly)
  elly <- ifelse(elly > ymax, ymax, elly)

  llines(ellx, elly, col='gray40')

}
#splom(~penguins[ , pengvars], upper.panel=splom_panel.ellipse, pscales=0)
#splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.ellipse, pscales=0)

# corrgram.R

#' Draw a correlogram
#'
#' Note: `corrgram()` uses base R graphics.
#' The corrgram function produces a graphical display of a correlation matrix,
#' called a correlogram.  The cells of the matrix can be shaded or colored to
#' show the correlation value. Automatic variable reordering can be used to
#' improve the visualization.
#'
#' Note: Use the 'col.regions' argument to specify colors.
#'
#' Non-numeric columns in the data will be ignored.
#'
#' The off-diagonal panels are specified with \code{panel.pts},
#' \code{panel.pie}, \code{panel.shade}, \code{panel.fill}, \code{panel.bar},
#' \code{panel.ellipse}, \code{panel.conf}. \code{panel.cor}.
#'
#' Diagonal panels are specified with \code{panel.txt}, \code{panel.minmax},
#' \code{panel.density}.
#'
#' Use a NULL panel to omit drawing the panel.
#'
#' This function is basically a modification of the \code{pairs.default}
#' function with the use of customized panel functions.
#'
#' The panel.conf function uses \code{cor.test} and calculates pearson
#' correlations.  Confidence intervals are not available in \code{cor.test} for
#' other methods (kendall, spearman).
#'
#' You can create your own panel functions by starting with one of the included
#' panel functions and making suitable modifications.  Note that because of the
#' way the panel functions are called inside the main function, your custom
#' panel function must include the arguments shown in the \code{panel.pts}
#' function, even if the custom panel function does not use those arguments!
#'
#' @aliases corrgram panel.bar panel.conf panel.cor panel.density panel.ellipse
#' panel.fill panel.minmax panel.pie panel.pts panel.shade panel.txt
#'
#' @param x A \emph{tall} data frame with one observation per row, or a
#' correlation matrix.
#'
#' @param type Use 'data' or 'cor'/'corr' to explicitly specify that 'x' is
#' data or a correlation matrix.  Rarely needed.
#'
#' @param order Should variables be re-ordered?
#' This will improve the visualization by placing similar variables next
#' to each other. Use TRUE or "PCA" for angular PCA-based
#' re-ordering (see Friendly, 2002).
#' If the 'seriation' package is installed, then any
#' distance-based method for \link[seriation]{seriate} can be used.
#' Examples are "OLO" for optimal leaf ordering,
#' "GW", "HC", and "MDS" (see Hahsler and Buchta, 2008).
#'
#' @param labels Labels to use (instead of data frame variable names) for
#' diagonal panels. If 'order' option is used, this vector of labels will be
#' also be appropriately reordered by the function.
#'
#' @param panel Function used to plot the contents of each panel.
#'
#' @param lower.panel,upper.panel Separate panel functions used below/above the
#' diagonal.
#'
#' @param diag.panel,text.panel Panel function used on the diagonal.
#'
#' @param label.pos Horizontal and vertical placement of label in diagonal
#' panels.
#'
#' @param label.srt String rotation for diagonal labels.
#'
#' @param cex.labels Graphics parameter for diagonal panels.
#'
#' @param row1attop TRUE for diagonal like " \ ", FALSE for diagonal like " / ".
#'
#' @param dir Use \code{dir="left"} instead of 'row1attop'.
#'
#' @param gap Distance between panels.
#'
#' @param abs Use absolute value of correlations for clustering?  Default FALSE.
#'
#' @param col.regions A \emph{function} returning a vector of colors.
#'
#' @param cor.method Correlation method to use in panel functions.  Default is
#' 'pearson'.  Alternatives: 'spearman', 'kendall'.
#'
#' @param outer.labels A list of the form 'list(bottom,left,top,right)'.
#' If 'bottom=TRUE' (for example), variable labels are added along the
#' bottom outside edge.
#'
#' For more control, use 'bottom=list(labels,cex,srt,adj)', where 'labels' is a vector
#' of variable labels, 'cex' affects the size, 'srt' affects the rotation, and 'adj'
#' affects the adjustment of the labels.
#' Defaults: 'labels' uses column names; cex=1';
#' 'srt=90' (bottom/top), 'srt=0' (left/right);
#' 'adj=1' (bottom/left), 'adj=0' (top/right).
#'
#' @param ... Additional arguments passed to plotting methods.
#'
#' @return The correlation matrix used for plotting is returned. The 'order' and 'abs'
#' arguments affect the returned value.
#'
#' @author Kevin Wright
#'
#' @references
#' Friendly, Michael.  2002.  Corrgrams: Exploratory Displays for
#' Correlation Matrices.  \emph{The American Statistician}, 56, 316--324.
#' \url{http://datavis.ca/papers/corrgram.pdf}
#'
#' D. J. Murdoch and E. D. Chow. 1996. A Graphical Display of Large Correlation
#' Matrices. The American Statistician, 50, 178-180.
#'
#' Hahsler M, Hornik K, Buchta C. 2008. Getting things in order: An
#' introduction to the R package seriation.
#' \emph{Journal of Statistical Software}, 25(3), 1--34.
#' ISSN 1548-7660, \doi{10.18637/jss.v025.i03}
#'
#' @keywords hplot
#'
#' @examples
#'
#' # To reproduce the figures in Michael Friendly's paper, see the
#' # vignette, or see the file 'friendly.r' in this package's
#' # test directory.
#'
#' # Demonstrate density panel, correlation confidence panel
#' corrgram(iris, lower.panel=panel.pts, upper.panel=panel.conf,
#'          diag.panel=panel.density)
#'
#' # Demonstrate panel.shade, panel.pie, principal component ordering
#' vars2 <- c("Assists","Atbat","Errors","Hits","Homer","logSal",
#'            "Putouts","RBI","Runs","Walks","Years")
#' corrgram(baseball[vars2], order=TRUE, main="Baseball data PC2/PC1 order",
#'          lower.panel=panel.shade, upper.panel=panel.pie)
#'
#' # Use reordering options from package seriation (only available if
#' #    package seriation is installed)
#' if ("seriation" %in% rownames(installed.packages())) {
#' corrgram(baseball[vars2], order= "OLO",
#'           main="Baseball data with optimal leaf order",
#'           lower.panel=panel.shade, upper.panel=panel.pie)
#'
#' corrgram(baseball[vars2], order= "R2E",
#'           main="Baseball data with rank 2 ellipse seriation",
#'           lower.panel=panel.shade, upper.panel=panel.pie)
#'
#' # the following seriation methods are available
#' seriation::list_seriation_methods("dist")
#' }
#'
#' # CAUTION: The latticeExtra package also has a 'panel.ellipse' function
#' # that clashes with the same-named function in corrgram. In order to use
#' # the right one, the example below uses 'lower.panel=corrgram::panel.ellipse'.
#' # If you do not have latticeExtra loaded, you can just use
#' # 'lower.panel=panel.ellipse'.
#'
#' # Demonstrate panel.bar, panel.ellipse, panel.minmax, col.regions
#' corrgram(auto, order=TRUE, main="Auto data (PC order)",
#'          lower.panel=corrgram::panel.ellipse,
#'          upper.panel=panel.bar, diag.panel=panel.minmax,
#'          col.regions=colorRampPalette(c("darkgoldenrod4", "burlywood1",
#'                                         "darkkhaki", "darkgreen")))
#'
#' # 'vote' is a correlation matrix, not a data frame
#' corrgram(vote, order=TRUE, upper.panel=panel.cor)
#'
#'
#' # outer labels, all options, larger margins, xlab, ylab
#' labs=colnames(state.x77)
#' corrgram(state.x77, oma=c(7, 7, 2, 2),
#'          outer.labels=list(bottom=list(labels=labs,cex=1.5,srt=60),
#'                            left=list(labels=labs,cex=1.5,srt=30,adj=c(1,0))))
#' mtext("Bottom", side=1, cex=2, line = -1.5, outer=TRUE, xpd=NA)
#' mtext("Left", side=2, cex=2, line = -1.5, outer=TRUE, xpd=NA)
#'
#' @import graphics
#' @import grDevices
#' @import stats
#' @importFrom utils installed.packages
#' @export corrgram
corrgram <- function (x, type=NULL,
            order=FALSE, labels, 
            panel = panel.shade,
            lower.panel = panel, 
            upper.panel = panel,
            diag.panel = NULL,
            text.panel = textPanel,
            label.pos = c(0.5, 0.5), label.srt=0,
            cex.labels = NULL,
            row1attop = TRUE, dir="", gap = 0,
            abs=FALSE,
            col.regions = colorRampPalette(c("red","salmon","white",
                                             "royalblue","navy")),
            cor.method="pearson",
            outer.labels=NULL,
            ...) {

  if(is.null(order)) order <- FALSE

  if( cor.method != "pearson" &&
      (deparse(substitute(lower.panel)) == "panel.conf" ||
       deparse(substitute(upper.panel)) == "panel.conf"))
    stop("'panel.conf' only allows 'pearson' method. Try 'panel.cor' instead.")

  if(length(label.pos) < 2) stop("label.pos needs a vector of length 2")

  # Direction
  if(dir=="") {
    if(row1attop) dir <- "left" else dir <- "right"
  }
  if (dir=="\\") dir <-  "left"
  if (dir=="/") dir <-  "right"

  if (ncol(x) < 2) stop("Only one column in the argument to 'corrgram'")

  # if a matrix x has only colnames, isSymmetric(x) reports FALSE
  # use unname(x) so it will report TRUE
  if(is.matrix(x) && isSymmetric(unname(x))) {
    maybeCorr <- TRUE
    if( min(x, na.rm=TRUE) < -1-.Machine$double.eps ) {
      warning("Matrix symmetric, but some values < -1. Treated as data.frame.")
      maybeCorr <- FALSE
    }
    if( max(x, na.rm=TRUE) >  1+.Machine$double.eps ) {
      warning("Matrix symmetric, but some values > +1. Treated as data.frame.")
      maybeCorr <- FALSE
    }
  } else maybeCorr <- FALSE

  if(is.null(type)){
    type <- if(maybeCorr) "corr" else "data"
  } else if(type=="data"){
    if(maybeCorr)
      warning("This looks like a correlation matrix.")
  } else if(type=="cor" || type=="corr") {
    type <- "corr"
    if(!maybeCorr)
      warning("This is NOT a correlation matrix.")
  } else {
    stop("unknown data type in 'corrgram'")
  }

  # Remove non-numeric columns from data frames
  if(type=="data" && !is.matrix(x)) x <- x[ , vapply(x, is.numeric, logical(1))]

  # If a data matrix, then calculate the correlation matrix
  if(type=="data") {
    cmat <- cor(x, use="pairwise.complete.obs", method=cor.method)
  } else {
    cmat <- x
  }

  # Save the correlation matrix for returning to user, re-ordered below
  cmat.return <- cmat

  # should we use absolute correlations for determining ordering?
  cmat <- if(abs) abs(cmat) else cmat


  # Default order
  if(order==FALSE) {
    ord <- 1:nrow(cmat)
  # Re-order the data to group highly correlated variables
  } else if(order==TRUE || order=="PC" || order=="PCA"){
    # Calculate the size of the angle between the horizontal axis and the
    # PC vectors
    x.eigen <- eigen(cmat)$vectors[,1:2]
    e1 <- x.eigen[,1]
    e2 <- x.eigen[,2]
    alpha <- ifelse(e1>0, atan(e2/e1), atan(e2/e1)+pi) # Friendly eqn 1
    ord <- order(alpha)
    x <- if(type=="data") x[,ord] else x[ord, ord]
    cmat.return <- cmat.return[ord,ord]
  } else {
    # Any other order method is delegated to seriation
    # The 'seriation' package is very heavy (many dependencies), so we
    # do not import it, but only check to see if it is installed.
    if (!("seriation" %in% rownames(installed.packages())))
      stop("Please use install.packages('seriation') for this 'order' option.")
    # "OLO" is used in this book
    # R Visualizations: Derive Meaning from Data By David Gerbing · 2020 p. 125
    distx <- as.dist(sqrt(1 - cmat))
    ss <- seriation::seriate(distx, method=order) # from seriation package
    ord <- seriation::get_order(ss)
    x <- if(type=="data") x[,ord] else x[ord,ord]
    cmat.return <- cmat.return[ord,ord]
  }

  textPanel <- function(x = 0.5, y = 0.5, txt, cex, srt) {
    text(x, y, txt, cex=cex, srt=srt)
  }

  # Don't pass some arguments on to the panel functions via the '...'
  localPlot <- function(..., main, oma, font.main, cex.main)
    plot(...)
  localLowerPanel <- function(..., main, oma, font.main, cex.main)
    lower.panel(...)
  localUpperPanel <- function(..., main, oma, font.main, cex.main)
    upper.panel(...)
  localDiagPanel <- function(..., main, oma, font.main, cex.main)
    diag.panel(...)

  dots <- list(...)
  nmdots <- names(dots)

  # Check for non-numeric data
  if (!is.matrix(x)) {
    x <- as.data.frame(x)
    for(i in seq(along=names(x))) {
      if(is.factor(x[[i]]) || is.logical(x[[i]]))
        x[[i]] <- as.numeric(x[[i]])
      if(!is.numeric(unclass(x[[i]])))
        stop("non-numeric argument to 'corrgram'")
    }
  } else if (!is.numeric(x)) stop("non-numeric argument to 'corrgram'")

  # Get panel functions
  panel <- match.fun(panel)
  if((has.lower <- !is.null(lower.panel)) && !missing(lower.panel))
    lower.panel <- match.fun(lower.panel)
  if((has.upper <- !is.null(upper.panel)) && !missing(upper.panel))
    upper.panel <- match.fun(upper.panel)

  has.diag  <- !is.null(diag.panel)
  if(has.diag && !missing( diag.panel))
    diag.panel <- match.fun( diag.panel)

  if(dir=="left") {
    tmp <- lower.panel
    lower.panel <- upper.panel
    upper.panel <- tmp
    tmp <- has.lower
    has.lower <- has.upper
    has.upper <- tmp
  }

  nc <- ncol(x)
  has.labs <- TRUE
  if (missing(labels)) {
    labels <- colnames(x)
    if (is.null(labels)) labels <- paste("var", 1:nc)
  } else if (order!=FALSE) {
      labels <- labels[ord]
  } else if(is.null(labels)) has.labs <- FALSE
  if(is.null(text.panel)) has.labs <- FALSE

  oma <- if("oma" %in% nmdots) dots$oma else NULL
  main <- if("main" %in% nmdots) dots$main else NULL

  # Plot layout

  if (is.null(oma)) {
    oma <- c(4, 4, 4, 4)
    if (!is.null(main)) oma[3] <- 6 # Space for the title
  }

  opar <- par(mfrow = c(nc, nc), mar = rep.int(gap/2, 4), oma = oma)
  on.exit(par(opar))

  # Main loop to draw each panel
  for (i in if(dir=="left") 1:nc else nc:1)
    for (j in 1:nc) {
      # Set up plotting area
      localPlot(x[, j], x[, i], xlab = "", ylab = "",
                axes = FALSE, type = "n", ...)

      if(i == j || (i < j && has.lower) || (i > j && has.upper) ) {

        if(i == j) {
          # Diagonal panel
          if (has.diag) {
            if(type=="data")
              localDiagPanel(as.vector(x[, i]), NULL, ...)
            else
              localDiagPanel(NULL, x[i,i], ...)
          }

          # Diagonal text
          if (has.labs) {
            par(usr = c(0, 1, 0, 1))
            if(is.null(cex.labels)) {
              l.wid <- strwidth(labels, "user")
              cex.labels <- max(0.8, min(2, .9 / max(l.wid)))
            }
            text.panel(label.pos[1], label.pos[2], labels[i],
                       cex = cex.labels, srt=label.srt)
          }
        } else if(i < j) {
          # Lower panel
          if(type=="data")
            localLowerPanel(as.vector(x[, j]), as.vector(x[, i]), NULL,
                            col.regions, cor.method, ...)
          else
            localLowerPanel(NULL, NULL, x[j,i], col.regions, cor.method, ...)
        } else {
          # Upper panel
          if(type=="data")
            localUpperPanel(as.vector(x[, j]), as.vector(x[, i]), NULL,
                            col.regions, cor.method, ...)
          else
            localUpperPanel(NULL, NULL, x[j,i], col.regions, cor.method, ...)
        }

      } else { # No panel drawn
        par(new = FALSE)
      }

    }

  if (!is.null(main)) {
    font.main <-
      if("font.main" %in% nmdots) dots$font.main else par("font.main")
    cex.main <- if("cex.main" %in% nmdots) dots$cex.main else par("cex.main")
    mtext(main, 3, 3, TRUE, 0.5, cex = cex.main, font = font.main)
  }

  # ----------------------------------------------------------------------------

  corrgram.outer.labels(1, nc, ord, labels, outer.labels$bottom)
  corrgram.outer.labels(2, nc, ord, labels, outer.labels$left)
  corrgram.outer.labels(3, nc, ord, labels, outer.labels$top)
  corrgram.outer.labels(4, nc, ord, labels, outer.labels$right)

  ## # Add overall labels
  ## mtext(xlab, side = 1, line = -1.5, outer=TRUE, xpd=NA)
  ## mtext(ylab, side = 2, line = -1.5, outer=TRUE, xpd=NA)

  invisible(cmat.return)
}

corrgram.outer.labels <- function(side,nc,ord,labels, ll){
  # side = 1,2,3,4
  # nc
  # ord = re-ordering of columns/rows
  # ll=list(labels,cex,las,srt, adj)
  # inspired by Leo Leopold, with modifications to rotate text from
  # http://menugget.blogspot.com/2014/08/rotated-axis-labels-in-r-plots.html

  if(is.null(ll)) return()
  if(isTRUE(ll)) ll <- list() # allow TRUE

  # In case text.panel=NULL, we need to set par(usr)
  par(usr = c(0, 1, 0, 1))

  # If no labels are specified, use the default column names
  if(is.null(ll$labels)) ll$labels = labels

  if(length(ll$labels) != nc)
    stop("The length of labels of side ", side, " does not match the number of columns of the corrgram.")

  # re-order if needed
  ll$labels <- ll$labels[ord]

  # default cex
  if(is.null(ll$cex)) ll$cex=1
  # default srt
  if((side==1 | side==3) && is.null(ll$srt)) ll$srt=90 # vert
  if((side==2 | side==4) && is.null(ll$srt)) ll$srt=0 # horiz
  # default adj
  if((side==1 | side==2) && is.null(ll$adj)) ll$adj=1
  if((side==3 | side==4) && is.null(ll$adj)) ll$adj=0


  for(i in 1:nc){
    # row/column grid position down/right
    if(side==1) { # bottom
      par(mfg=c(nc, i))
      # without 'clip', only the first label is added
      clip(0, -2, 0, 1)
      text(x=0.5, y= 0 - 0.05*(1-0), labels=ll$labels[i],
           cex=ll$cex, srt=ll$srt, adj=ll$adj, xpd=NA)
    } else if (side==2){ # left
      par(mfg=c(i, 1))
      clip(0, -2, 0, 1)
      text(x=0 - 0.05*(1-0), y=0.5, labels=ll$labels[i],
           cex=ll$cex, srt=ll$srt, adj=ll$adj, xpd=NA)
    } else if (side==3) { # top
      par(mfg=c(1, i))
      clip(0, -2, 0, 1)
      text(x=0.5, y= 1 + 0.05*(1-0), labels=ll$labels[i],
           cex=ll$cex, srt=ll$srt, adj=ll$adj, xpd=NA)
    } else if (side==4) { # right
      par(mfg=c(i, nc))
      clip(0, -2, 0, 1)
      text(x=1 + .05*(1-0), y=0.5, labels=ll$labels[i],
           cex=ll$cex, srt=ll$srt, adj=ll$adj, xpd=NA)
    }

  }

  return()
}

# ----------------------------------------------------------------------------
# Panel functions

#' @export
panel.pts <- function(x, y, corr=NULL, col.regions, cor.method, ...){

  # For correlation matrix, do nothing
  if(!is.null(corr)) return()

  plot.xy(xy.coords(x, y), type="p", ...)
  box(col="lightgray")
}

#' @export
panel.pie <- function(x, y, corr=NULL, col.regions, cor.method, ...){

  # if corr not given, try to calculate it
  if(is.null(corr)) {
    if(sum(complete.cases(x,y)) < 2) {
      warning("Need at least 2 complete cases for cor()")
      return()
    } else {
      corr <- cor(x, y, use='pair', method=cor.method)
    }
  }

  # coordinates of box
  usr <- par()$usr  # par is in graphics package
  xmin <- usr[1]
  xmax <- usr[2]
  ymin <- usr[3]
  ymax <- usr[4]
  # multiply the radius by .96 so the circles do not overlap
  rx <- (xmax-xmin)/2 * .96
  ry <- (ymax-ymin)/2 * .96
  centerx <- (xmin+xmax)/2
  centery <- (ymin+ymax)/2

  # draw circle
  numseg <- 60
  angles <- seq(from=0, to=2*pi, length.out=numseg)
  circ <- cbind(centerx + cos(angles)*rx, centery + sin(angles)*ry)
  lines(circ[,1], circ[,2], col='gray40',...)

  n_bins <- 14
  pal <- col.regions(n_bins)
  col.ind <- as.numeric(cut(corr, breaks=seq(from=-1, to=1, length.out=n_bins+1),
                            include.lowest=TRUE))
  col.pie <- pal[col.ind]

  # overlay a colored pie piece
  numseg <- round(60*abs(corr),0)
  if(numseg>0){ # Watch out for the case with 0 numseg
    angles <- seq(from=pi/2, to=pi/2+(2*pi* -corr), length.out=numseg)
    circ <- cbind(centerx + cos(angles)*rx, centery + sin(angles)*ry)
    circ <- rbind(circ, c(centerx, centery), circ[1,])
    polygon(circ[,1], circ[,2], col=col.pie,border="gray40")
  }

}

#' @export
panel.shade <- function(x, y, corr=NULL, col.regions, cor.method, ...){

  # If corr not given, try to calculate it
  if(is.null(corr)) {
    if(sum(complete.cases(x,y)) < 2) {
      warning("Need at least 2 complete cases for cor()")
      return()
    } else {
      corr <- cor(x, y, use='pair', method=cor.method)
    }
  }

  n_bins <- 14
  pal <- col.regions(n_bins)
  col.ind <- as.numeric(cut(corr, breaks=seq(from=-1, to=1, length.out=n_bins+1),
                            include.lowest=TRUE))
  usr <- par("usr")
  # Solid fill
  rect(usr[1], usr[3], usr[2], usr[4], col=pal[col.ind], border=NA)
  # Add diagonal lines
  if(!is.na(corr)) {
    rect(usr[1], usr[3], usr[2], usr[4], density=5,
         angle=ifelse(corr>0, 45, 135), col="white")
  }
  # Boounding box needs to plot on top of the shading, so do it last.
  box(col='lightgray')
}

#' @export
panel.ellipse <- function(x,y, corr=NULL, col.regions, cor.method, ...){

  # For correlation matrix, do nothing
  if(!is.null(corr)) return()

  # If too few points, do nothing
  if(sum(complete.cases(x,y)) < 2) {
    warning("Need at least 2 complete cases to draw ellipse.")
    return()
  }

  # Draw an ellipse. Use df numerator = 2, and df denominator = n-1, as in friendly's code.
  dfn <- 2 # df numerator
  dfd <- length(x)-1
  shape <- var(cbind(x,y),na.rm=TRUE)
  keep <- (!is.na(x) & !is.na(y))
  center <- c(mean(x[keep]),mean(y[keep]))
  radius <- sqrt(dfn*qf(.68,dfn,dfd))
  numseg <- 180
  angles <- seq(from=0, to=2*pi, length.out=numseg)
  unit.circle <- cbind(cos(angles),sin(angles))
  ellipse.pts <- t(center+radius*t(unit.circle%*%chol(shape)))
  ellx <- ellipse.pts[,1]
  elly <- ellipse.pts[,2]
  # Truncate ellipse at min/max or at bounding box
  usr <- par()$usr
  xmin <- usr[1]
  xmax <- usr[2]
  ymin <- usr[3]
  ymax <- usr[4]
  ellx <- ifelse(ellx < xmin, xmin, ellx)
  ellx <- ifelse(ellx > xmax, xmax, ellx)
  elly <- ifelse(elly < ymin, ymin, elly)
  elly <- ifelse(elly > ymax, ymax, elly)
  lines(ellx, elly, col='gray40',...)

  # Add a lowess line through the ellipse.  Use 'ok' to remove NAs
  ok <- is.finite(x) & is.finite(y)
  if (any(ok))
    lines(stats::lowess(x[ok], y[ok], f = 2/3, iter = 3),
          col = "red", ...)
}

#' @export
panel.bar <- function(x, y, corr=NULL, col.regions, cor.method, ...){
  # Use 'bars' as in Friendly, figure 1

  usr <- par()$usr
  xmin <- usr[1]
  xmax <- usr[2]
  ymin <- usr[3]
  ymax <- usr[4]

  if (is.null(corr)) {
    if(sum(complete.cases(x,y)) < 2) {
      warning("Need at least 2 complete cases for cor()")
      return()
    } else {
      corr <- cor(x, y, use = "pairwise.complete.obs", method=cor.method)
    }
  }

  n_bins <- 14
  pal <- col.regions(n_bins)
  col.ind <- as.numeric(cut(corr, breaks = seq(from = -1, to = 1,
                                    length.out = n_bins + 1), include.lowest = TRUE))
  col.bar <- pal[col.ind]
  if(corr < 0) {
    # Draw up from bottom
    ymax <- ymin + (ymax-ymin) *  abs(corr)
    rect(xmin, ymin, xmax, ymax, col = pal[col.ind],
         border = "lightgray")
  } else if (corr > 0){
    # Draw down from top
    ymin <- ymax - (ymax-ymin)*corr
    rect(xmin, ymin, xmax, ymax, col = pal[col.ind],
         border = "lightgray")
  }

}

#' @export
panel.fill <- function(x, y, corr=NULL, col.regions, cor.method, ...){

  # If corr not given, try to calculate it
  if(is.null(corr)) {
    if(sum(complete.cases(x,y)) < 2) {
      warning("Need at least 2 complete cases for cor()")
      return()
    } else {
      corr <- cor(x, y, use='pair', method=cor.method)
    }
  }

  n_bins <- 14
  pal <- col.regions(n_bins)
  col.ind <- as.numeric(cut(corr, breaks=seq(from=-1, to=1, length.out=n_bins+1),
                            include.lowest=TRUE))
  usr <- par("usr")
  # Solid fill
  rect(usr[1], usr[3], usr[2], usr[4], col=pal[col.ind], border=NA)

  # Boounding box needs to plot on top of the shading, so do it last.
  box(col='lightgray')
}


#' @export
panel.cor <- function(x, y, corr=NULL, col.regions, cor.method, digits=2,
                      cex.cor, ...){
  # Correlation values only, colored

  # If corr not given, try to calculate it
  if(is.null(corr)) {
    if(sum(complete.cases(x,y)) < 2) {
      warning("Need at least 2 complete cases for cor()")
      return()
    } else {
      corr <- cor(x, y, use='pair', method=cor.method)
    }
  }

  auto <- missing(cex.cor)
  usr <- par("usr")
  on.exit(par(usr))
  par(usr = c(0, 1, 0, 1))

  n_bins <- 14
  pal <- col.regions(n_bins)
  col.ind <- as.numeric(cut(corr, breaks=seq(from=-1, to=1, length.out=n_bins+1),
                            include.lowest=TRUE))

  # determine string width using absolute values so that
  # negative numbers are not wider than positive numbers
  abscorr <- formatC(abs(corr), digits=digits, format='f')
  corr <- formatC(corr, digits=digits, format='f')
  if(auto) cex.cor <- 0.7/strwidth(abscorr)
  text(0.5, 0.5, corr, cex=cex.cor, col=pal[col.ind])

}

#' @export
panel.conf <- function(x, y, corr=NULL, col.regions, cor.method, digits=2,
                       cex.cor, ...){

  auto <- missing(cex.cor)
  usr <- par("usr")
  on.exit(par(usr))
  par(usr = c(0, 1, 0, 1))

  # n_bins <- 14
  # pal <- col.regions(n_bins)

  # For correlation matrix, only show the correlation
  if(!is.null(corr)) {
    est <- corr
    # ignore signs for calculating string width
    absest <- formatC(abs(est), digits=digits, format='f')
    est <- formatC(est, digits=digits, format='f')
    if(auto) cex.cor <- 0.7/strwidth(absest)
    text(0.5, 0.6, est, cex=cex.cor) #, col=pal[col.ind])

  } else { # Calculate correlation and confidence interval
    if(sum(complete.cases(x,y)) < 4) {
      warning("Need at least 4 complete cases for cor.test()")
    } else {
      results <- cor.test(x, y, alternative = "two.sided", method=cor.method)
      # point estimate
      est <- results$estimate
      absest <- formatC(abs(est), digits=digits, format='f')
      est <- formatC(est, digits=digits, format='f')
      if(auto) cex.cor <- 0.7/strwidth(absest)
      text(0.5, 0.6, est, cex=cex.cor) #, col=pal[col.ind])
      # confidence interval, only for pearson
      ci <- results$conf.int
      ci <- formatC(ci, digits=2, format='f')
      ci <- paste0("(",ci[1],",",ci[2],")")
      if(auto) cex.cor <- 0.8/strwidth(ci)
      text(0.5, 0.3, ci, cex=cex.cor) # , col=pal[col.ind])
    }

  }
}

#' @export
panel.txt <- function(x=0.5, y=0.5, txt, cex, srt){
  text(x, y, txt, cex=cex, srt=srt)
}

#' @export
panel.density <- function(x, corr=NULL, ...){
  # For correlation matrix, do nothing
  if(!is.null(corr)) return()

  dd = density(x, na.rm=TRUE)
  xr=range(dd$x)
  yr=range(dd$y)
  par(usr = c(min(xr), max(xr), min(yr), max(yr)*1.1))
  plot.xy(xy.coords(dd$x, dd$y), type="l", col="black", ...)
  box(col="lightgray")
}

#' @export
panel.minmax <- function(x, corr=NULL, ...){
  # For correlation matrix, do nothing
  if(!is.null(corr)) return()
  # Put the minimum in the lower-left corner and the
  # maximum in the upper-right corner
  xmin <- round(min(x, na.rm=TRUE),2)
  xmax <- round(max(x, na.rm=TRUE),2)
  text(xmin, xmin, xmin, cex=1, adj=c(0,0))
  text(xmax, xmax, xmax, cex=1, adj=c(1,1))
}

# -----

if(FALSE) {

# Exploring ideas for adding a legend

# layout doesn't work because par(mfrow=) is incompatible

# -------------
# https://stackoverflow.com/questions/13081310/combining-multiple-complex-plots-as-panels-in-a-single-figure/14567298#14567298

# this works but is intolerably slow for 10x10

library(gridGraphics)
library(grid)

grab_grob <- function(){
  grid.echo()
  grid.grab()
}

arr <- replicate(4,
                 matrix(runif(n=9),nrow=3,ncol=3), simplify = FALSE)

library(gplots)
gl <- lapply(1:4, function(i){
  corrgram(arr[[i]])
  grab_grob()
})

grid.newpage()
library(gridExtra)
grid.arrange(grobs=gl, ncol=2, clip=TRUE)

}

# corrgram_grid.R

# For a correlation matrix, grid_panel.ellipse, grid_panel.pie do nothing.

# Currently there are 14 color bins, but this could be made an argument.
# The legend also has 14 bins, which is not ideal.

#' Draw a correlogram using `grid` graphics
#'
#' `corrgram2()` produces a correlogram using grid graphics. The off-diagonal cells
#' can be shaded or filled with custom panel functions to show the correlation
#' structure of a data matrix or correlation matrix.
#'
#' This function is a grid-graphics variant of [corrgram()]. It accepts either a
#' data matrix/data frame with one observation per row or a correlation matrix.
#' When raw data are supplied, correlations are computed with
#' `use = "pairwise.complete.obs"`.
#'
#' Variable reordering can be used to improve the display by placing related
#' variables near each other. `order = TRUE` and `order = "PC"` use the
#' PCA-based angular ordering described by Friendly (2002). `order =
#' "seriation"` uses `cba::seriation(..., method = "Optimal")` and requires the
#' `cba` package.
#'
#' @aliases corrgram2 grid_panel.conf grid_panel.ellipse grid_panel.fill
#' grid_panel.pie grid_panel.pts grid_panel.shade grid_text.panel
#' 
#' @param x A data frame or matrix with one observation per row, or a
#'   correlation matrix.
#' 
#' @param type Use 'data' or 'cor'/'corr' to explicitly specify whether
#'   `x` is raw data or a correlation matrix. Usually this is inferred.
#' 
#' @param order Should variables be reordered? Use `FALSE` for no reordering,
#'   `TRUE` or `"PC"` for PCA-based angular ordering, or `"seriation"` for
#'   optimal seriation via the `cba` package.
#' 
#' @param labels Labels to use on the diagonal instead of column names.
#' 
#' @param panel Default panel function used for both `lower.panel` and
#'   `upper.panel`.
#' 
#' @param ... Additional arguments passed to the panel functions.
#' 
#' @param lower.panel,upper.panel Separate panel functions used below and above
#'   the diagonal.
#' 
#' @param diag.panel Optional panel function used on the diagonal before drawing
#'   diagonal labels.
#' 
#' @param text.panel Included for API compatibility with [corrgram()]. Diagonal
#'   labels are currently drawn with [grid_text.panel()].
#' 
#' @param label.pos Horizontal and vertical placement of the diagonal label.
#' 
#' @param label.srt Rotation for diagonal labels.
#' 
#' @param cex.labels Label size for diagonal labels. Use `"fit"` to size labels
#'   to the panel width.
#' 
#' @param dir Direction of the main diagonal. Use `"left"` or `"\\"` for a
#'   descending diagonal, and `"right"` or `"/"` for an ascending diagonal.
#' 
#' @param legend If `TRUE`, draw a legend for the color scale.
#' 
#' @param col.regions A function returning a vector of colors.
#' 
#' @param cor.method Correlation method passed to panel functions. Default is
#'   `"pearson"`.
#' 
#' @param title Optional title drawn above the correlogram.
#' 
#' @param abs Logical; if `TRUE`, use absolute correlations for variable
#'   reordering.
#'
#' @return Invisibly returns `NULL`.
#'
#' @references
#' Friendly, Michael. 2002. Corrgrams: Exploratory Displays for Correlation
#' Matrices. *The American Statistician*, 56, 316--324.
#' \url{http://datavis.ca/papers/corrgram.pdf}
#'
#' D. J. Murdoch and E. D. Chow. 1996. A Graphical Display of Large Correlation
#' Matrices. The American Statistician, 50, 178-180.
#'
#' @examples
#' # Draw a grid-based correlogram from data
#' vars6 <- setdiff(colnames(auto), c("Model", "Origin"))
#' corrgram2(auto[vars6], order = TRUE,
#'     lower.panel = grid_panel.shade,
#'     upper.panel = grid_panel.pie)
#'
#' # 'vote' is a correlation matrix
#' corrgram2(vote, order = TRUE, upper.panel = grid_panel.conf)
#'
#' @keywords hplot
#' @import grid
#' @export
corrgram2 <- function(x,
                type=NULL,
                order=FALSE, 
                labels,
                panel=grid_panel.shade,
                ...,
                lower.panel = panel,
                upper.panel = panel,
                diag.panel = NULL,
                text.panel = grid_text.panel,
                label.pos = c(0.5, 0.5),
                label.srt=0,
                cex.labels="fit",
                dir="left", 
                legend=FALSE,
                col.regions=colorRampPalette(c("red","salmon","white","royalblue","navy")),
                cor.method="pearson",
                title=NULL,
                abs=FALSE) {
    
  # Direction
  if (dir=="\\") dir <-  "left"
  if (dir=="/") dir <-  "right"
  
  if (ncol(x) < 2) stop("Only one column in the argument to 'corrgram'")
  
  # if a matrix x has only colnames, isSymmetric(x) reports FALSE
  # use unname(x) so it will report TRUE
  if(is.matrix(x) && isSymmetric(unname(x))) {
    maybeCorr <- TRUE
    if( min(x, na.rm=TRUE) < -1-.Machine$double.eps ) {
      warning("Matrix symmetric, but some values < -1. Treated as data.frame.")
      maybeCorr <- FALSE
    }
    if( max(x, na.rm=TRUE) >  1+.Machine$double.eps ) {
      warning("Matrix symmetric, but some values > +1. Treated as data.frame.")
      maybeCorr <- FALSE
    }
  } else maybeCorr <- FALSE

  if(is.null(type)){
    type <- if(maybeCorr) "corr" else "data"
  } else if(type=="data"){
    if(maybeCorr)
      warning("This looks like a correlation matrix.")
  } else if(type=="cor" || type=="corr") {
    type <- "corr"
    if(!maybeCorr)
      warning("This is NOT a correlation matrix.")
  } else {
    stop("unknown data type in 'corrgram'")
  }

  # Remove non-numeric columns from data frames
  if(type=="data" && !is.matrix(x)) x <- x[ , vapply(x, is.numeric, logical(1))]

   # If a data matrix, then calculate the correlation matrix
  if(type=="data") {
    cmat <- cor(x, use="pairwise.complete.obs", method=cor.method)
  } else {
    cmat <- x
  }

  # Save the correlation matrix for returning to user, re-ordered below
  cmat.return <- cmat

  # should we use absolute correlations for determining ordering?
  cmat <- if(abs) abs(cmat) else cmat


 
  # If a data matrix, then calculate the correlation matrix
  x.cor <- if(type=="data") cor(x, use="pairwise.complete.obs") else x
  
  has.labs <- TRUE
  if (missing(labels)) {
    labels <- colnames(x)
    if (is.null(labels)) labels <- paste("var", 1:ncol(x))
  }
  else if(is.null(labels)) has.labs <- FALSE
  

  # default order
  if(order==FALSE){
    ord <- 1:nrow(cmat)
    # Re-order the data to group highly correlated variables
  } else if(order==TRUE || order=="PC" || order=="PCA"){
    # calculate the size of the angle between the horizontal axis and the
    # PC vectors
    x.eigen <- eigen(cmat)$vectors[,1:2]
    e1 <- x.eigen[,1]
    e2 <- x.eigen[,2]
    alpha <- ifelse(e1>0, atan(e2/e1), atan(e2/e1)+pi)
    ord <- order(alpha)
    x <- if(type=="data") x[,ord] else x[ord, ord]
    cmat.return <- cmat.return[ord, ord]
  } else {
    # Any other order method is delegated to seriation
    # The 'seriation' package is very heavy (many dependencies), so we
    # do not import it, but only check to see if it is installed.
    if (!("seriation" %in% rownames(installed.packages())))
      stop("Please use install.packages('seriation') for this 'order' option.")
    # "OLO" is used in this book
    # R Visualizations: Derive Meaning from Data By David Gerbing · 2020 p. 125
    distx <- as.dist(sqrt(1 - cmat))
    ss <- seriation::seriate(distx, method=order) # from seriation package
    ord <- seriation::get_order(ss)
    x <- if(type=="data") x[,ord] else x[ord,ord]
    cmat.return <- cmat.return[ord,ord]
  }
 
  # Don't pass some arguments on to the panel functions
  localLowerPanel <- function(..., main, oma, font.main, cex.main)
    lower.panel(...)
  localUpperPanel <- function(..., main, oma, font.main, cex.main)
    upper.panel(...)
  localDiagPanel <- function(..., main, oma, font.main, cex.main)
    diag.panel(...)
  #  localTextPanel <- function(...)
  
  #dots <- list(...)
  #nmdots <- names(dots)
   
  nc <- ncol(x)
   
  # New page
  grid::grid.newpage()
  grid::grid.rect(gp=grid::gpar(fill="light blue"))

  if((has.lower <- !is.null(lower.panel)) && !missing(lower.panel))
    lower.panel <- match.fun(lower.panel)
  if((has.upper <- !is.null(upper.panel)) && !missing(upper.panel))
    upper.panel <- match.fun(upper.panel)
  
  has.diag  <- !is.null(diag.panel)
  if(has.diag && !missing( diag.panel))
    diag.panel <- match.fun( diag.panel)
  
  if(dir=="left") {
    tmp <- lower.panel; lower.panel <- upper.panel; upper.panel <- tmp
    tmp <- has.lower; has.lower <- has.upper; has.upper <- tmp
  }
  
  # overall
  pushViewport(plotViewport(margins=c(1.1,1.1,3.1,1.1)))
  # Title
  if(!is.null(title))
    grid.text(title, x=unit(0.5, "npc"), 
                     y = unit(1, "npc") + unit(1.5, "lines"),
                     gp = gpar(fontsize = 16))
  
  # Splom. respect=TRUE keeps the panels square
  scatplot <- grid.layout(nc, nc, respect=TRUE)
  
  # Legend
  if(legend==TRUE){
    # Keep space for legend
    pushViewport(viewport(layout=grid.layout(1,2,widths=unit(c(.8,.2),
                                                             c("npc","npc")))))
    
    pushViewport(plotViewport(layout=scatplot,
                              layout.pos.col=1, layout.pos.row=1))
    grid.rect(gp=gpar(fill="blue"))
  } else {
    # No legend
    pushViewport(plotViewport(margins=c(1.1,1.1,4.1,2.1), layout=scatplot))
  }
  
  grid::grid.rect(gp=grid::gpar(fill="gray95"))
  #  grid.show.layout(scatplot)
  for(i in 1:nc) {
    for(j in 1:nc){
      
      xvals <- if(type=="data") as.vector(x[,j]) else NULL
      yvals <- if(type=="data") as.vector(x[,i]) else NULL
      corr <- if(type=="corr") x[j,i] else NULL
      
      colpos <- ifelse(dir=="left", j, nc-j+1)
      if(type=="data")
        pushViewport(dataViewport(xvals, yvals,
                                  layout.pos.row=i, layout.pos.col=colpos))
      else
        pushViewport(viewport(layout.pos.row=i, layout.pos.col=colpos))
      
      if(i==j) {
        # Diagonal panel (non-text)
        if (has.diag) {
          localDiagPanel(xvals, ...)
        }
        
        # Diagonal text
        if (has.labs) {
          if(cex.labels=="fit") {
            tw <- convertWidth(max(stringWidth(labels)), "inches",
                               valueOnly=TRUE)
            vw <- convertWidth(unit(1, "npc"), "inches", valueOnly=TRUE)
            cex.labels <- vw/tw
          }
          grid_text.panel(label.pos[1], label.pos[2], txt=labels[i], cex=cex.labels, rot=label.srt)
        }
        
      } else if (i<j) {
        # Lower panel
        if(type=="data")
          localLowerPanel(xvals, yvals, NULL, col.regions, cor.method, ...)
        else
          localLowerPanel(NULL, NULL, corr, col.regions, cor.method, ...)
      } else {
        # Upper panel
        if(type=="data")
          localUpperPanel(xvals, yvals, NULL, col.regions, cor.method, ...)
        else
          localUpperPanel(NULL, NULL, corr, col.regions, cor.method, ...)            
      }
      popViewport() # Panel
    }
  }
  popViewport() # Scatter plot
  
  # Draw legend at right
  if(legend) {
    n.colors <- 14
    legplot <- grid.layout(n.colors, 2)
    pushViewport(plotViewport(margins=c(1,1,1,3),
                              layout=legplot, layout.pos.row=1, layout.pos.col=2))
    #grid.rect(gp=gpar(fill="yellow"))
    pal <- col.regions(n.colors)
    midpoints <- seq(-1, 1, length.out=n.colors+1)[-1] - diff(seq(-1, 1, length.out=n.colors+1))[1]/2
    for(i in 1:n.colors) {
      # colored cells between break points
      pushViewport(viewport(layout.pos.row=i, layout.pos.col=1))
      grid.rect(gp=gpar(fill=pal[i]))
      popViewport()
      # labels for break points
      pushViewport(viewport(layout.pos.row=i, layout.pos.col=2))
      #grid.text(midpoints[i], 2, just="left", gp=gpar(cex=0.5))
      #grid.text(midpoints[i])
      grid.text(formatC(midpoints[i], format = "f", digits = 2))
      popViewport()
    }

    popViewport() # For legend
  }
  
  popViewport() # Entire graph

  invisible(cmat.return)
}


# ----------------------------------------------------------------------------

#' @export
grid_panel.fill <- function(x, y, corr=NULL, col.regions, cor.method="pearson") {
  # Same as grid_panel.shade, but without diagonal lines

  # if corr not given, try to calculate it
  if(is.null(corr)) {
    if(sum(complete.cases(x,y)) < 2) {
      warning("Need at least 2 complete cases for cor()")
      return()
    } else {
      corr <- cor(x, y, use='pair', method=cor.method)
    }
  }
  
  n_bins <- 14
  pal <- col.regions(n_bins)
  col.ind <- as.numeric(cut(corr, breaks=seq(from=-1, to=1, length.out=n_bins+1),
                            include.lowest=TRUE))
#browser()
  # Solid fill
  grid.rect(gp=gpar(col=pal[col.ind], fill=pal[col.ind]))
}
#corrgram2(auto[, vars6], abs=TRUE, panel=grid_panel.fill)

#' @export
grid_panel.shade <- function(x, y, corr=NULL, col.regions, cor.method="pearson") {
    # If corr not given, try to calculate it
  if(is.null(corr)) {
    if(sum(complete.cases(x,y)) < 2) {
      warning("Need at least 2 complete cases for cor()")
      return()
    } else {
      corr <- cor(x, y, use='pair', method=cor.method)
    }
  }

  n_bins <- 14
  pal <- col.regions(n_bins)
  col.ind <- as.numeric(cut(corr, breaks=seq(from=-1, to=1, length.out=n_bins+1),
                            include.lowest=TRUE))
  
  # Solid fill
  grid.rect(gp=gpar(col="lightgray", fill=pal[col.ind]))
  # Add diagonal lines in white
  if(corr>0){
    grid.polyline(c(0,1,.5,1,0,.5), c(0,1,0,.5,.5,1), id=c(1,1,2,2,3,3),
                  gp=gpar(col="white"))
  } else if(corr<0) {
    grid.polyline(c(1,0,.5,0,1,.5), c(0,1,0,.5,.5,1), id=c(1,1,2,2,3,3),
                  gp=gpar(col="white"))
  }
  
  # Bounding box needs to plot on top of the shading, so do it last.
  grid.rect(gp=gpar(col="lightgray", fill=NA))
}
#corrgram2(auto[, vars6], abs=TRUE, panel=grid_panel.shade)

#' @export
grid_panel.pts <- function(x, y, corr=NULL, col.regions, cor.method="pearson", ...){
  if(!is.null(corr)) return()
  # use hollow circles for the points
  grid.points(x, y, pch=1, gp = gpar(cex=0.5, fill=NA))
  grid.rect(gp=gpar(col="lightgray",fill=NA))
}
#corrgram2(auto[, vars6], abs=TRUE, panel=grid_panel.pts) # broken 

  
#' @export
grid_panel.pie <- function(x, y, corr=NULL, col.regions, cor.method="pearson"){

  # if corr not given, try to calculate it
  if(is.null(corr)) {
    if(sum(complete.cases(x,y)) < 2) {
      warning("Need at least 2 complete cases for cor()")
      return()
    } else {
      corr <- cor(x, y, use='pair', method=cor.method)
    }
  }
  
  # Start with the 'pie' piece
  # Multiply the radius by .97 so the circles do not overlap
  rad <- 0.5 * .97

  n_bins <- 14
  pal <- col.regions(n_bins)
  col.ind <- as.numeric(cut(corr, breaks=seq(from=-1, to=1, length.out=n_bins+1),
                            include.lowest=TRUE))
  col.pie <- pal[col.ind]
  
  numseg <- round(60*abs(corr),0)
  if(numseg>0){ # Watch out for the case with 0 numseg
    angles <- seq(pi/2, pi/2+(2*pi* -corr), length=numseg)
    circx <- c(0.5 + cos(angles)*rad, 0.5)
    circy <- c(0.5 + sin(angles)*rad, 0.5)
    grid.polygon(unit(circx, "npc"), unit(circy, "npc"),
                 gp=gpar(col=col.pie, fill=col.pie))
  }   
  
  # overlay the circle
  numseg <- 60
  angles <- seq(0, 2*pi, length=numseg)
  circx <- 0.5 + cos(angles)*rad
  circy <- 0.5 + sin(angles)*rad
  grid.polygon(unit(circx, "npc"), unit(circy, "npc"),
               gp=gpar(col="gray30", fill=NA))
  
}
#corrgram2(auto[, vars6], abs=TRUE, panel=grid_panel.pie) # NOT right fixme

#' @export
grid_panel.conf <- function(x, y, corr=NULL, col.regions, cor.method="pearson", ...){

  # if corr not given, try to calculate it
  if(is.null(corr)) {
    if(sum(complete.cases(x,y)) < 2) {
      warning("Need at least 2 complete cases for cor()")
      return()
    } else {
      corr <- cor(x, y, use='pair', method=cor.method)
    }
  }
  
  n_bins <- 14
  pal <- col.regions(n_bins)
  col.ind <- as.numeric(cut(corr, breaks=seq(from=-1, to=1, length.out=n_bins+1),
                            include.lowest=TRUE))
  col.pie <- pal[col.ind]
  
  if(is.null(x)) { # No data x, cannot calculate conf int
    grid.text(formatC(corr, 2, format='f'), gp=gpar(col=pal[col.ind]))      
  } else {
    if (corr < 1 - 1e-09) {
      conf.level <- .95
      z <- atanh(corr)
      b <- qnorm((1 - conf.level)/2)/sqrt(length(x) - 3)
      ci.z <- c(z + b, z - b)
      confint <- tanh(ci.z)
      confCol <- "black"
      if(confint[1]>0 | confint[2]<0) confCol <- pal[col.ind]
      confint <- formatC(confint, 2, format='f')
      confint <- paste("(",confint[1], ",", confint[2], ")",sep="")
    } else {
      confint <- "(NA,1)"
    }
    grid.text(formatC(corr, 2, format='f'), gp=gpar(col=pal[col.ind]))      
    grid.text(confint, y=.25, gp=gpar(col=confCol, cex=.5))
  }
}
#corrgram2(auto[, vars6], abs=TRUE, panel=grid_panel.conf)

#' @export
grid_panel.ellipse <- function(x, y, corr=NULL, col.regions, cor.method="pearson", ...){

  # For correlation matrix, do nothing
  if(!is.null(corr)) return()

  # If too few points, do nothing
  if(sum(complete.cases(x,y)) < 2) {
    warning("Need at least 2 complete cases to draw ellipse.")
    return()
  }

  # Min/Max of bounding box
  xrng <- extendrange(x, f=.05)
  yrng <- extendrange(y, f=.05)
  # Remove missing values
  keep <- (!is.na(x) & !is.na(y))
  x <- x[keep]
  y <- y[keep]
  # Draw an ellipse
  dfn <- 2
  dfd <- length(x)-1
  shape <- var(cbind(x,y))
  center <- c(mean(x),mean(y))
  radius <- sqrt(dfn*qf(.68,dfn,dfd))
  segments <- 75
  angles <- seq(0,2*pi,length=segments)
  unit.circle <- cbind(cos(angles),sin(angles))
  ellipse.pts <- t(center+radius*t(unit.circle%*%chol(shape)))
  ellx <- ellipse.pts[,1]
  elly <- ellipse.pts[,2]
  # Truncate ellipse at bounding box
  ellx <- ifelse(ellx < xrng[1], xrng[1], ellx) 
  ellx <- ifelse(ellx > xrng[2], xrng[2], ellx)
  elly <- ifelse(elly < yrng[1], yrng[1], elly)
  elly <- ifelse(elly > yrng[2], yrng[2], elly)
  grid.lines(unit(ellx, "native"), unit(elly, "native"),
             gp=gpar(col="gray30"))
  
  # Add a lowess line through the ellipse (if we have data)
  if(!is.null(x)){
    low <- stats::lowess(x, y, f=2/3, iter=3)
    grid.lines(unit(low$x,"native"), unit(low$y,"native"),
               gp=gpar(col="red"))
  }
  
}
#corrgram2(auto[, vars6], abs=TRUE, panel=grid_panel.ellipse)

#' @export
grid_text.panel <- function(x=0.5, y=0.5, txt, cex=1, rot=0){
  # diagonal panel labels
  grid.text(txt, x=x, y=y, just=c(.5,.5), rot=rot, gp=gpar(cex=cex))
}

# ----------------------------------------------------------------------------


if(FALSE){
  #data(auto, package="corrgram")
  #data(vote, package="corrgram")
  library(corrgram)
  vars6 <- setdiff(colnames(auto), c("Model", "Origin"))
  
  # compare speed of corrgram and corrgram2. 
  # grid version is much faster in Positron, but not in Rgui. 
  corrgram::corrgram(auto[,vars6], panel=corrgram::panel.pts)
  corrgram2(auto[, vars6], abs=TRUE, panel=grid_panel.pts)
  
  corrgram2(auto[, vars6], abs=TRUE, panel=grid_panel.fill, title="auto data")
  corrgram2(auto[, vars6], abs=TRUE, panel=grid_panel.shade)
  corrgram2(auto[, vars6], abs=TRUE, panel=grid_panel.pie)
  corrgram2(auto[, vars6], abs=TRUE, panel=grid_panel.conf)
  corrgram2(auto[, vars6], abs=TRUE, panel=grid_panel.ellipse)
  
  corrgram2(vote,abs=TRUE)
  corrgram2(vote,abs=TRUE, labels=1:12)
  corrgram2(vote,abs=TRUE, label.srt=0)
  corrgram2(vote,abs=TRUE, label.srt=60, cex.labels=1.5)
  corrgram2(vote,abs=TRUE,order=TRUE, legend=TRUE)
  corrgram2(vote, order="OLO", lower.panel=grid_panel.pie)
  corrgram2(vote, panel=grid_panel.conf)
  corrgram2(vote, panel=grid_panel.ellipse)
  corrgram2(vote, lower.panel=grid_panel.fill)
  corrgram2(vote, panel=grid_panel.pie, order=TRUE)
  corrgram2(vote, panel=grid_panel.pts) # fixme should err
  corrgram2(vote, upper.panel=grid_panel.shade)
  corrgram2(vote, lower.panel=grid_panel.fill, upper.panel=grid_panel.shade)
    
}

require(corrgram)

vars6 <- setdiff(colnames(auto), c("Model", "Origin"))

test_that("corrgram2 dataframe panel functions do not err", {

  corrgram2(auto[, vars6], panel=grid_panel.fill, title="auto data")
  corrgram2(auto[, vars6], panel=grid_panel.shade)
  corrgram2(auto[, vars6], panel=grid_panel.pie)
  corrgram2(auto[, vars6], panel=grid_panel.conf)
  corrgram2(auto[, vars6], panel=grid_panel.ellipse)

})

test_that("corrgram2 matrix panel functions do not err", {

  corrgram2(vote)
  corrgram2(vote, abs=TRUE, labels=1:12, label.srt=30, cex.labels=1.5)
  corrgram2(vote, abs=TRUE, order=TRUE, legend=TRUE)
  corrgram2(vote, panel=grid_panel.conf)
  corrgram2(vote, panel=grid_panel.ellipse) # does nothing
  corrgram2(vote, lower.panel=grid_panel.fill)
  corrgram2(vote, panel=grid_panel.pie, order=TRUE)
  corrgram2(vote, upper.panel=grid_panel.shade)
  corrgram2(vote, lower.panel=grid_panel.fill, upper.panel=grid_panel.shade)

})

test_that("corrgram2 vote with OLO order does not err", {
  skip_if_not_installed("seriation")
  corrgram2(vote, order="OLO", lower.panel=grid_panel.pie, 
  title="vote data with OLO order", cex.labels="fit", label.srt=45)
})

# corrgram_levelplot.R

#' Panel functions for lattice corrgrams via levelplot
#'
#' These functions provide custom panel methods for \code{lattice::levelplot()}.
#'
#' @param x Numeric coordinates from levelplot.
#' @param y Numeric coordinates from levelplot.
#' @param z Correlation values from levelplot.
#' @param subscripts Subscripts for lattice panel. (not used)
#' @param at Breaks for color levels.
#' @param level Confidence level for ellipse (default 0.9).
#' @param label Logical; if TRUE, show correlation values as text.
#' @param ... Additional arguments passed to panel functions.
#'
#' @details
#' 
#' \code{levelplot_panel.ellipse} Draws ellipses representing correlation coefficients
#' in the upper triangle of a matrix. Optionally adds numeric labels in the 
#' lower triangle.
#' 
#' \code{levelplot_panel.pie} Draws pie glyphs representing correlation coefficients,
#' omitting the diagonal.
#' 
#' @rdname levelplot_panel.ellipse
#' @importFrom ellipse ellipse
#' @importFrom lattice levelplot level.colors panel.polygon panel.text
#' @examples
#' library(lattice)
#' levelplot(vote, at = do.breaks(c(-1.01, 1.01), 20),
#'             xlab = NULL, ylab = NULL, colorkey = list(space = "top"),
#'             scales = list(x = list(rot = 90)), 
#'             panel = levelplot_panel.ellipse, label = TRUE)
#' levelplot(vote, at = do.breaks(c(-1.01, 1.01), 20),
#'             xlab = NULL, ylab = NULL, colorkey = list(space = "top"),
#'             scales = list(x = list(rot = 90)), 
#'             panel = levelplot_panel.pie, label = TRUE)
#' @export
levelplot_panel.ellipse <- function(
  x, y, z, subscripts, 
  at, level = 0.9, label = FALSE, ...) {
  x <- as.numeric(x)[subscripts]
  y <- as.numeric(y)[subscripts]
  z <- as.numeric(z)[subscripts]
  zcol <- level.colors(z, at = at, ...)

  # Draw the glyph for each correlation value
  for (i in seq(along = z)) {
    # Only dry glyphs for the upper triangle
    if(x[i] < y[i]) {
      xy.ell <- ellipse::ellipse(z[i], level = level, npoints = 50,
                   scale = c(.2, .2), centre = c(x[i], y[i]))
      panel.polygon(xy.ell, col = zcol[i], border = zcol[i], ...)
    }
  }

  if (label) {
    ixskip <- x <= y # only label the lower triangle
    panel.text(x = x[!ixskip], y = y[!ixskip],
               lab = 100 * round(z[!ixskip], 2), cex = 0.8,
               col = ifelse(z[!ixskip] < 0, "red", "navy"))
  }
}

# --------------------------------------------------------------------

#' @param scale Numeric; scaling factor for pie size (default 0.8).
#' 
#' @rdname levelplot_panel.ellipse
#' 
#' @export
levelplot_panel.pie <- function(x, y, z, subscripts, at = pretty(z), scale = 0.9, ...) {
  # x,y are coordinates of the matrix, z is the correlation value
  x <- as.numeric(x)[subscripts]
  y <- as.numeric(y)[subscripts]
  z <- as.numeric(z)[subscripts]

  zcol <- level.colors(z, at = at, ...)

  for (i in seq(along = z)) {
  if(x[i] == y[i]) next # skip the diagonal
    # The full circle
  grid::grid.circle(x = x[i], y = y[i], r = .5 * scale, default.units = "native")

  # pie-shaped piece for each correlation value
    numseg <- round(60*abs(z[i]),0) # number of segments for each pie piece
    if(numseg>0){ # Watch out for the case with 0 segments
      angles <- seq(pi/2, pi/2+(2*pi* -z[i]), length=numseg)
      xy.pie <- cbind(x[i] + cos(angles)*0.5*scale,
                      y[i] + sin(angles)*0.5*scale)
      xy.pie <- rbind(xy.pie, c(x[i], y[i]), xy.pie[1,])
      panel.polygon( xy.pie, col=zcol[i])
    }
  }
}

# data.R

#' Statistics of 1979 automobile models
#' 
#' Statistics for 74 automobiles in the 1979 model year as sold in the US.
#' 
#' The data is from various sources, primarily \emph{Consumer Reports}, April,
#' 1979, and the United States government EPA statistics on fuel consumption.
#' 
#' @format
#' A data frame with 74 observations on the following 14 variables.
#' \describe{ 
#' \item{Model}{Make and model of car.}
#' \item{Origin}{a factor with levels A,E,J}
#' \item{Price}{Price in dollars.} 
#' \item{MPG}{Miles per gallon.} 
#' \item{Rep78}{Repair record for 1978 on 1 (worst) to 5 (best) scale.} 
#' \item{Rep77}{Repair record for 1978 on 1 to 5 scale.}
#' \item{Hroom}{Headroom in inches.} 
#' \item{Rseat}{Rear seat clearance in inches.} 
#' \item{Trunk}{Trunk volume in cubic feet.}
#' \item{Weight}{Weight in pounds.} 
#' \item{Length}{Length in inches.} 
#' \item{Turn}{Turning diameter in feet.}
#' \item{Displa}{Engine displacement in cubic inches.}
#' \item{Gratio}{Gear ratio for high gear.} 
#' }
#' 
#' @source 
#' This data frame was created from
#' http://euclid.psych.yorku.ca/ftp/sas/sssg/data/auto.sas
#' 
#' @references 
#' Originally published in Chambers, Cleveland, Kleiner, and Tukey,
#' \emph{Graphical Methods for Data Analysis}, 1983, pages 352-355.
#' 
#' @examples
#' 
#' corrgram(auto[, -c(1:2)])
#' 
"auto"




#' Baseball Hitter's Data
#' 
#' Data are for 322 Major Leaque Baseball regular and substitute hitters in
#' 1986.
#' 
#' The levels of the player's positions have been collapsed to fewer levels for
#' a simpler analysis.  See the original data for the full list of positions.
#'  
#' The salary data were taken from Sports Illustrated, April 20, 1987.  The
#' salary of any player not included in that article is listed as an NA.  The
#' 1986 and career statistics were taken from The 1987 Baseball Encyclopedia
#' Update published by Collier Books, Macmillan Publishing Company, New York.
#' 
#' @format A data frame with 322 observations on the following 22 variables.
#' \describe{ 
#' \item{Name}{The hitter/player's name}
#' \item{League}{Player's league (American/National) at the beginning
#' of 1987} 
#' \item{Team}{Player's team at the beginning of 1987}
#' \item{Position}{Player's position in 1986: 1B=First base, 
#' 2B=Second base, 3B=Third base, C=Catcher, OF=Outfild, DH=Designated hitter,
#' SS=Short stop, UT=Utility} 
#' \item{Atbat}{Number of times at bat in 1986}
#' \item{Hits}{Number of hits in 1986} 
#' \item{Homer}{Number of home runs in 1986} 
#' \item{Runs}{Number of runs in 1986}
#' \item{RBI}{Runs batted in during 1986} 
#' \item{Walks}{Number of walks in 1986} 
#' \item{Years}{Number of years in the major leagues}
#' \item{Atbatc}{Number of times at bat in his career}
#' \item{Hitsc}{Number of hits in career} 
#' \item{Homerc}{Number of home runs in career} 
#' \item{Runsc}{Number of runs in career}
#' \item{RBIc}{Number of Runs Batted In in career}
#' \item{Walksc}{Number of walks in career}
#' \item{Putouts}{Number of putouts in 1986}
#' \item{Assists}{Number of assists in 1986}
#' \item{Errors}{Number of errors in 1986} 
#' \item{Salary}{Annual salary (in thousands) on opening day 1987} 
#' \item{logSal}{Log of salary} }
#' 
#' @source 
#' The data was originally published for the 1988 ASA Statistical
#' Graphics and Computing Data Exposition:
#' http://lib.stat.cmu.edu/data-expo/1988.html.
#' 
#' The version of the data used to create this data was found at
#' http://euclid.psych.yorku.ca/ftp/sas/sssg/data/baseball.sas
#'
#' @references 
#' Michael Friendly (2002). Corrgrams: Exploratory Displays for Correlation 
#' Matrices, \emph{The American Statistician}, Vol 56.
#' 
#' @examples
#' 
#' vars2 <- c("Assists","Atbat","Errors","Hits","Homer","logSal",
#'            "Putouts","RBI","Runs","Walks","Years")
#' corrgram(baseball[,vars2],
#'          lower.panel=panel.shade, upper.panel=panel.pie)
#' 
"baseball"





#' Voting correlations
#' 
#' Voting correlations
#' 
#' These are the correlations of traits, where each trait is measured for 17 
#' developed countries (Europe, US, Japan, Australia, New Zealand).
#' 
#' @format A 12x12 matrix.
#' 
#' @source 
#' Torben Iversen and David Soskice (2006). Electoral institutions and
#' the politics of coalitions: Why some democracies redistribute more than
#' others.  \emph{American Political Science Review}, 100, 165-81.  Table A2.
#' 
#' @references 
#' Using Graphs Instead of Tables.
#' http://tables2graphs.com/doku.php?id=03_descriptive_statistics
#' 
#' @examples
#' 
#' corrgram(vote, order=TRUE)
#' 
"vote"

require(corrgram)
require(lattice)

test_that("lattice levelplot panel functions do not err", {

  # vote data is in corrgram
  levelplot(vote, at = do.breaks(c(-1.01, 1.01), 20),
            xlab = NULL, ylab = NULL, colorkey = list(space = "top"),
            scales = list(x = list(rot = 90)), 
            panel = levelplot_panel.ellipse, label = TRUE)
  levelplot(vote, at = do.breaks(c(-1.01, 1.01), 20),
            xlab = NULL, ylab = NULL, colorkey = list(space = "top"),
            scales = list(x = list(rot = 90)), 
            panel = levelplot_panel.pie, label = TRUE)

})

test_that("lattice splom panel functions do not err", {

  pengvars <- c("bill_len", "bill_dep", "flipper_len", "body_mass")
  splom(~penguins[ , pengvars], upper.panel=splom_panel.pie, pscales=0)
  splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.pie, pscales=0)
  splom(~penguins[ , pengvars], upper.panel=splom_panel.shade, pscales=0)
  splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.shade, pscales=0)
  splom(~penguins[ , pengvars], upper.panel=splom_panel.ellipse, pscales=0)
  splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.ellipse, pscales=0)

})

---
title: "Examples of grid corrgrams"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Examples of grid corrgrams}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## Package overview

The `corrgram` package provides functions for creating corrgrams using three different graphics systems, base, grid, and lattice.

Base R graphics
+ single function `corrgram()` for dataframes or matrices.
+ Enables most features found in the paper by @friendly2002corrgrams.
- No automatic legend.
- Not easily combined with other graphics.

`lattice` graphics
+ Separate panel functions for `lattice::levelplot()` for dataframes and `lattice::splom()` for correlation matrices.
+ Enables automatic legend.
+ Enables corrgrams conditioned on other variables.
+ Can be combined with other lattice graphics for complex figures.
- Not feature complete compared to base R.

`grid` graphics
+ single function `corrgram2()` for either dataframes or correlation matrices. 
+ Enables automatic legend.
+ Can be combined with other grid graphics for complex figures.
- Not feature complete compared to base R.
+ Faster than base R when evaluated inside Positron.

## This vignette

This vignette demonstrates how to create corrgrams using `grid` graphics with the `corrgram2()` function and a variety of panel functions for visualizing correlations in different ways.

```{r setup, include=FALSE}
library(corrgram)
```

## Grid Panels for corrgram2

This vignette demonstrates the use of grid-based panels in `corrgram2`, which provide flexible and modern correlation matrix visualizations.

## Correlation matrix corrgram in grid

The `vote` dataset contains roll call voting records for US Senators. Here we show a grid-based correlation plot with absolute correlations, ordering, and a legend.

```{r vote-example, fig.width=6, fig.height=6}
corrgram2(vote, abs = TRUE, order = TRUE, legend = TRUE, title = "vote data")
```

## Dataframe corrgram in grid

The `auto` dataset contains various automobile attributes. We select a subset of numeric variables and display a grid-based correlation plot using the fill panel.

```{r auto-example, fig.width=6, fig.height=6}
vars6 <- setdiff(colnames(auto), c("Model", "Origin"))
corrgram2(auto[, vars6], 
          lower.panel = grid_panel.shade, upper.panel=grid_panel.pie,
          title = "auto data", legend = TRUE)
```

---
title: "Examples of lattice corrgrams"
author: "Kevin Wright"
date: "`r Sys.Date()`"
bibliography: corrgram.bib
output: 
  rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Examples of lattice corrgrams}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## Package overview

The `corrgram` package provides functions for creating corrgrams using three different graphics systems, base, grid, and lattice.

Base R graphics
+ single function `corrgram()` for dataframes or matrices.
+ Enables most features found in the paper by @friendly2002corrgrams.
- No automatic legend.
- Not easily combined with other graphics.

`lattice` graphics
+ Separate panel functions for `lattice::levelplot()` for dataframes and `lattice::splom()` for correlation matrices.
+ Enables automatic legend.
+ Enables corrgrams conditioned on other variables.
+ Can be combined with other lattice graphics for complex figures.
- Not feature complete compared to base R.

`grid` graphics
+ single function `corrgram2()` for either dataframes or correlation matrices. 
+ Enables automatic legend.
+ Can be combined with other grid graphics for complex figures.
- Not feature complete compared to base R.
+ Faster than base R when evaluated inside Positron.

## This vignette

This vignette demonstrates how to create corrgrams using `lattice` graphics, you can use some custom panel functions provided in the `corrgram` package along with the `lattice::splom()` or `lattice::levelplot()` functions. An example of each type of corrgram is shown below.

## Correlation matrix corrgram in lattice

The `levelplot()` function in lattice only has a single plotting region, so does not (by default) suppot upper and lower panels. However, you can write a custom panel function with different glyphs above and below the diagonal. See the `panel.ellipse()` example below.

Using `splom()` makes it easy to include a color scale next to the corrgram.

```{r corrgram-lattice-correlation-matrix, fig.width=6, fig.height=6}
library("lattice")
library("corrgram")

# The easiest way to have an automatic color key is to set the theme
opar <- trellis.par.get()
trellis.par.set(
  regions=list(col=colorRampPalette(c("red","salmon","white","royalblue","navy")))
)

# Create a correlation matrix
library(MASS) # foor Cars93
cor.Cars93 <-   cor(Cars93[, !sapply(Cars93, is.factor)], use = "pair")
ord <- order.dendrogram(as.dendrogram(hclust(dist(cor.Cars93))))
cars93 <- cor.Cars93[ord,ord]
head(cars93)

# lattice corrgram using pie-shaped glyphs
levelplot(cars93, xlab = NULL, ylab = NULL,
          at = do.breaks(c(-1.01, 1.01), 101), panel = levelplot_panel.pie,
          scales = list(x = list(rot = 90)), colorkey = list(space = "top") )

# lattice corrgram using ellipse-shaped glyphs above the diagonal
# and value labels below the diagonal
levelplot(cars93, xlab = NULL, ylab = NULL,
          at = do.breaks(c(-1.01, 1.01), 101), panel = levelplot_panel.ellipse,
          label=TRUE,
          scales = list(x = list(rot = 90)), colorkey = list(space = "top") )

```

## Scatterplot matrix corrgram in lattice

Since the `lattice::splom()` function supports conditioning on a factor, we can use it to create corrgrams that are conditioned on a factor.

The penguins data provides a nice example of Simpson's paradox, where the overall correlation between two variables can be negative, but the correlation within each group (species) can be positive. 

```{r}
pengvars <- c("bill_len", "bill_dep", "flipper_len", "body_mass")
library(lattice)
splom(~penguins[ , pengvars], upper.panel=splom_panel.pie, pscales=0)
splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.pie, pscales=0)
splom(~penguins[ , pengvars], upper.panel=splom_panel.shade, pscales=0)
splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.shade, pscales=0)
splom(~penguins[ , pengvars], upper.panel=splom_panel.ellipse, pscales=0)
splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.ellipse, pscales=0)
```

You can also use the `hexbin` package to create hexagonal binning plots in the upper panels of the scatterplot matrix.  This is useful when you have a large number of points and want to visualize the density of points in different regions of the plot.
```{r}
# Hexbin
library(lattice)
library(hexbin)
splom(~penguins[ , pengvars], upper.panel=hexbin::panel.hexbinplot, pscales=0)
splom(~penguins[ , pengvars]|penguins$species, upper.panel=hexbin::panel.hexbinplot, pscales=0)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{vote}
\alias{vote}
\title{Voting correlations}
\format{
A 12x12 matrix.
}
\source{
Torben Iversen and David Soskice (2006). Electoral institutions and
the politics of coalitions: Why some democracies redistribute more than
others.  \emph{American Political Science Review}, 100, 165-81.  Table A2.
}
\usage{
vote
}
\description{
Voting correlations
}
\details{
These are the correlations of traits, where each trait is measured for 17 
developed countries (Europe, US, Japan, Australia, New Zealand).
}
\examples{

corrgram(vote, order=TRUE)

}
\references{
Using Graphs Instead of Tables.
http://tables2graphs.com/doku.php?id=03_descriptive_statistics
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{auto}
\alias{auto}
\title{Statistics of 1979 automobile models}
\format{
A data frame with 74 observations on the following 14 variables.
\describe{ 
\item{Model}{Make and model of car.}
\item{Origin}{a factor with levels A,E,J}
\item{Price}{Price in dollars.} 
\item{MPG}{Miles per gallon.} 
\item{Rep78}{Repair record for 1978 on 1 (worst) to 5 (best) scale.} 
\item{Rep77}{Repair record for 1978 on 1 to 5 scale.}
\item{Hroom}{Headroom in inches.} 
\item{Rseat}{Rear seat clearance in inches.} 
\item{Trunk}{Trunk volume in cubic feet.}
\item{Weight}{Weight in pounds.} 
\item{Length}{Length in inches.} 
\item{Turn}{Turning diameter in feet.}
\item{Displa}{Engine displacement in cubic inches.}
\item{Gratio}{Gear ratio for high gear.} 
}
}
\source{
This data frame was created from
http://euclid.psych.yorku.ca/ftp/sas/sssg/data/auto.sas
}
\usage{
auto
}
\description{
Statistics for 74 automobiles in the 1979 model year as sold in the US.
}
\details{
The data is from various sources, primarily \emph{Consumer Reports}, April,
1979, and the United States government EPA statistics on fuel consumption.
}
\examples{

corrgram(auto[, -c(1:2)])

}
\references{
Originally published in Chambers, Cleveland, Kleiner, and Tukey,
\emph{Graphical Methods for Data Analysis}, 1983, pages 352-355.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{baseball}
\alias{baseball}
\title{Baseball Hitter's Data}
\format{
A data frame with 322 observations on the following 22 variables.
\describe{ 
\item{Name}{The hitter/player's name}
\item{League}{Player's league (American/National) at the beginning
of 1987} 
\item{Team}{Player's team at the beginning of 1987}
\item{Position}{Player's position in 1986: 1B=First base, 
2B=Second base, 3B=Third base, C=Catcher, OF=Outfild, DH=Designated hitter,
SS=Short stop, UT=Utility} 
\item{Atbat}{Number of times at bat in 1986}
\item{Hits}{Number of hits in 1986} 
\item{Homer}{Number of home runs in 1986} 
\item{Runs}{Number of runs in 1986}
\item{RBI}{Runs batted in during 1986} 
\item{Walks}{Number of walks in 1986} 
\item{Years}{Number of years in the major leagues}
\item{Atbatc}{Number of times at bat in his career}
\item{Hitsc}{Number of hits in career} 
\item{Homerc}{Number of home runs in career} 
\item{Runsc}{Number of runs in career}
\item{RBIc}{Number of Runs Batted In in career}
\item{Walksc}{Number of walks in career}
\item{Putouts}{Number of putouts in 1986}
\item{Assists}{Number of assists in 1986}
\item{Errors}{Number of errors in 1986} 
\item{Salary}{Annual salary (in thousands) on opening day 1987} 
\item{logSal}{Log of salary} }
}
\source{
The data was originally published for the 1988 ASA Statistical
Graphics and Computing Data Exposition:
http://lib.stat.cmu.edu/data-expo/1988.html.

The version of the data used to create this data was found at
http://euclid.psych.yorku.ca/ftp/sas/sssg/data/baseball.sas
}
\usage{
baseball
}
\description{
Data are for 322 Major Leaque Baseball regular and substitute hitters in
1986.
}
\details{
The levels of the player's positions have been collapsed to fewer levels for
a simpler analysis.  See the original data for the full list of positions.
 
The salary data were taken from Sports Illustrated, April 20, 1987.  The
salary of any player not included in that article is listed as an NA.  The
1986 and career statistics were taken from The 1987 Baseball Encyclopedia
Update published by Collier Books, Macmillan Publishing Company, New York.
}
\examples{

vars2 <- c("Assists","Atbat","Errors","Hits","Homer","logSal",
           "Putouts","RBI","Runs","Walks","Years")
corrgram(baseball[,vars2],
         lower.panel=panel.shade, upper.panel=panel.pie)

}
\references{
Michael Friendly (2002). Corrgrams: Exploratory Displays for Correlation 
Matrices, \emph{The American Statistician}, Vol 56.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corrgram_levelplot.R
\name{levelplot_panel.ellipse}
\alias{levelplot_panel.ellipse}
\alias{levelplot_panel.pie}
\title{Panel functions for lattice corrgrams via levelplot}
\usage{
levelplot_panel.ellipse(
  x,
  y,
  z,
  subscripts,
  at,
  level = 0.9,
  label = FALSE,
  ...
)

levelplot_panel.pie(x, y, z, subscripts, at = pretty(z), scale = 0.9, ...)
}
\arguments{
\item{x}{Numeric coordinates from levelplot.}

\item{y}{Numeric coordinates from levelplot.}

\item{z}{Correlation values from levelplot.}

\item{subscripts}{Subscripts for lattice panel. (not used)}

\item{at}{Breaks for color levels.}

\item{level}{Confidence level for ellipse (default 0.9).}

\item{label}{Logical; if TRUE, show correlation values as text.}

\item{...}{Additional arguments passed to panel functions.}

\item{scale}{Numeric; scaling factor for pie size (default 0.8).}
}
\description{
These functions provide custom panel methods for \code{lattice::levelplot()}.
}
\details{
\code{levelplot_panel.ellipse} Draws ellipses representing correlation coefficients
in the upper triangle of a matrix. Optionally adds numeric labels in the 
lower triangle.

\code{levelplot_panel.pie} Draws pie glyphs representing correlation coefficients,
omitting the diagonal.
}
\examples{
library(lattice)
levelplot(vote, at = do.breaks(c(-1.01, 1.01), 20),
            xlab = NULL, ylab = NULL, colorkey = list(space = "top"),
            scales = list(x = list(rot = 90)), 
            panel = levelplot_panel.ellipse, label = TRUE)
levelplot(vote, at = do.breaks(c(-1.01, 1.01), 20),
            xlab = NULL, ylab = NULL, colorkey = list(space = "top"),
            scales = list(x = list(rot = 90)), 
            panel = levelplot_panel.pie, label = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corrgram_splom.R
\name{splom_panel.pie}
\alias{splom_panel.pie}
\alias{splom_panel.shade}
\alias{splom_panel.ellipse}
\title{Panel functions for lattice corrgrams via splom}
\usage{
splom_panel.pie(
  x,
  y,
  z,
  subscripts,
  at = pretty(z),
  cor.method = "pearson",
  ...
)

splom_panel.shade(
  x,
  y,
  z,
  subscripts,
  at = pretty(z),
  col.regions = NULL,
  cor.method = "pearson",
  ...
)

splom_panel.ellipse(x, y, col.regions, cor.method = "pearson", ...)
}
\arguments{
\item{x}{Numeric coordinates from levelplot.}

\item{y}{Numeric coordinates from levelplot.}

\item{z}{Correlation values from levelplot.}

\item{subscripts}{Subscripts for lattice panel. (not used)}

\item{at}{Breaks for color levels.}

\item{cor.method}{Correlation method (default "pearson").}

\item{...}{Additional arguments passed to panel functions.}

\item{col.regions}{Color palette for shading (default NULL uses internal
red white blue palette).}
}
\description{
These functions provide custom panel methods for \code{lattice::splom()}.
}
\details{
\code{splom_panel.pie} Draws pie glyphs representing correlation coefficients,
omitting the diagonal.

\code{splom_panel.shade} Draws shaded rectangles with hash lines representing correlation
coefficients.

\code{splom_panel.ellipse} Draws ellipses representing correlation coefficients.
The position of the ellipse is determined by the position of the data,
and the shape of the ellipse is determined by the correlation.
}
\examples{
library(lattice)
pengvars <- c("bill_len", "bill_dep", "flipper_len", "body_mass")
splom(~penguins[ , pengvars], upper.panel=splom_panel.pie, pscales=0)
splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.pie, pscales=0)
splom(~penguins[ , pengvars], upper.panel=splom_panel.shade, pscales=0)
splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.shade, pscales=0)
splom(~penguins[ , pengvars], upper.panel=splom_panel.ellipse, pscales=0)
splom(~penguins[ , pengvars]|penguins$species, upper.panel=splom_panel.ellipse, pscales=0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corrgram_grid.R
\name{corrgram2}
\alias{corrgram2}
\alias{grid_panel.conf}
\alias{grid_panel.ellipse}
\alias{grid_panel.fill}
\alias{grid_panel.pie}
\alias{grid_panel.pts}
\alias{grid_panel.shade}
\alias{grid_text.panel}
\title{Draw a correlogram using `grid` graphics}
\usage{
corrgram2(
  x,
  type = NULL,
  order = FALSE,
  labels,
  panel = grid_panel.shade,
  ...,
  lower.panel = panel,
  upper.panel = panel,
  diag.panel = NULL,
  text.panel = grid_text.panel,
  label.pos = c(0.5, 0.5),
  label.srt = 0,
  cex.labels = "fit",
  dir = "left",
  legend = FALSE,
  col.regions = colorRampPalette(c("red", "salmon", "white", "royalblue", "navy")),
  cor.method = "pearson",
  title = NULL,
  abs = FALSE
)
}
\arguments{
\item{x}{A data frame or matrix with one observation per row, or a
correlation matrix.}

\item{type}{Use 'data' or 'cor'/'corr' to explicitly specify whether
`x` is raw data or a correlation matrix. Usually this is inferred.}

\item{order}{Should variables be reordered? Use `FALSE` for no reordering,
`TRUE` or `"PC"` for PCA-based angular ordering, or `"seriation"` for
optimal seriation via the `cba` package.}

\item{labels}{Labels to use on the diagonal instead of column names.}

\item{panel}{Default panel function used for both `lower.panel` and
`upper.panel`.}

\item{...}{Additional arguments passed to the panel functions.}

\item{lower.panel, upper.panel}{Separate panel functions used below and above
the diagonal.}

\item{diag.panel}{Optional panel function used on the diagonal before drawing
diagonal labels.}

\item{text.panel}{Included for API compatibility with [corrgram()]. Diagonal
labels are currently drawn with [grid_text.panel()].}

\item{label.pos}{Horizontal and vertical placement of the diagonal label.}

\item{label.srt}{Rotation for diagonal labels.}

\item{cex.labels}{Label size for diagonal labels. Use `"fit"` to size labels
to the panel width.}

\item{dir}{Direction of the main diagonal. Use `"left"` or `"\\"` for a
descending diagonal, and `"right"` or `"/"` for an ascending diagonal.}

\item{legend}{If `TRUE`, draw a legend for the color scale.}

\item{col.regions}{A function returning a vector of colors.}

\item{cor.method}{Correlation method passed to panel functions. Default is
`"pearson"`.}

\item{title}{Optional title drawn above the correlogram.}

\item{abs}{Logical; if `TRUE`, use absolute correlations for variable
reordering.}
}
\value{
Invisibly returns `NULL`.
}
\description{
`corrgram2()` produces a correlogram using grid graphics. The off-diagonal cells
can be shaded or filled with custom panel functions to show the correlation
structure of a data matrix or correlation matrix.
}
\details{
This function is a grid-graphics variant of [corrgram()]. It accepts either a
data matrix/data frame with one observation per row or a correlation matrix.
When raw data are supplied, correlations are computed with
`use = "pairwise.complete.obs"`.

Variable reordering can be used to improve the display by placing related
variables near each other. `order = TRUE` and `order = "PC"` use the
PCA-based angular ordering described by Friendly (2002). `order =
"seriation"` uses `cba::seriation(..., method = "Optimal")` and requires the
`cba` package.
}
\examples{
# Draw a grid-based correlogram from data
vars6 <- setdiff(colnames(auto), c("Model", "Origin"))
corrgram2(auto[vars6], order = TRUE,
    lower.panel = grid_panel.shade,
    upper.panel = grid_panel.pie)

# 'vote' is a correlation matrix
corrgram2(vote, order = TRUE, upper.panel = grid_panel.conf)

}
\references{
Friendly, Michael. 2002. Corrgrams: Exploratory Displays for Correlation
Matrices. *The American Statistician*, 56, 316--324.
\url{http://datavis.ca/papers/corrgram.pdf}

D. J. Murdoch and E. D. Chow. 1996. A Graphical Display of Large Correlation
Matrices. The American Statistician, 50, 178-180.
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corrgram_base.R
\name{corrgram}
\alias{corrgram}
\alias{panel.bar}
\alias{panel.conf}
\alias{panel.cor}
\alias{panel.density}
\alias{panel.ellipse}
\alias{panel.fill}
\alias{panel.minmax}
\alias{panel.pie}
\alias{panel.pts}
\alias{panel.shade}
\alias{panel.txt}
\title{Draw a correlogram}
\usage{
corrgram(
  x,
  type = NULL,
  order = FALSE,
  labels,
  panel = panel.shade,
  lower.panel = panel,
  upper.panel = panel,
  diag.panel = NULL,
  text.panel = textPanel,
  label.pos = c(0.5, 0.5),
  label.srt = 0,
  cex.labels = NULL,
  row1attop = TRUE,
  dir = "",
  gap = 0,
  abs = FALSE,
  col.regions = colorRampPalette(c("red", "salmon", "white", "royalblue", "navy")),
  cor.method = "pearson",
  outer.labels = NULL,
  ...
)
}
\arguments{
\item{x}{A \emph{tall} data frame with one observation per row, or a
correlation matrix.}

\item{type}{Use 'data' or 'cor'/'corr' to explicitly specify that 'x' is
data or a correlation matrix.  Rarely needed.}

\item{order}{Should variables be re-ordered?
This will improve the visualization by placing similar variables next
to each other. Use TRUE or "PCA" for angular PCA-based
re-ordering (see Friendly, 2002).
If the 'seriation' package is installed, then any
distance-based method for \link[seriation]{seriate} can be used.
Examples are "OLO" for optimal leaf ordering,
"GW", "HC", and "MDS" (see Hahsler and Buchta, 2008).}

\item{labels}{Labels to use (instead of data frame variable names) for
diagonal panels. If 'order' option is used, this vector of labels will be
also be appropriately reordered by the function.}

\item{panel}{Function used to plot the contents of each panel.}

\item{lower.panel, upper.panel}{Separate panel functions used below/above the
diagonal.}

\item{diag.panel, text.panel}{Panel function used on the diagonal.}

\item{label.pos}{Horizontal and vertical placement of label in diagonal
panels.}

\item{label.srt}{String rotation for diagonal labels.}

\item{cex.labels}{Graphics parameter for diagonal panels.}

\item{row1attop}{TRUE for diagonal like " \ ", FALSE for diagonal like " / ".}

\item{dir}{Use \code{dir="left"} instead of 'row1attop'.}

\item{gap}{Distance between panels.}

\item{abs}{Use absolute value of correlations for clustering?  Default FALSE.}

\item{col.regions}{A \emph{function} returning a vector of colors.}

\item{cor.method}{Correlation method to use in panel functions.  Default is
'pearson'.  Alternatives: 'spearman', 'kendall'.}

\item{outer.labels}{A list of the form 'list(bottom,left,top,right)'.
If 'bottom=TRUE' (for example), variable labels are added along the
bottom outside edge.

For more control, use 'bottom=list(labels,cex,srt,adj)', where 'labels' is a vector
of variable labels, 'cex' affects the size, 'srt' affects the rotation, and 'adj'
affects the adjustment of the labels.
Defaults: 'labels' uses column names; cex=1';
'srt=90' (bottom/top), 'srt=0' (left/right);
'adj=1' (bottom/left), 'adj=0' (top/right).}

\item{...}{Additional arguments passed to plotting methods.}
}
\value{
The correlation matrix used for plotting is returned. The 'order' and 'abs'
arguments affect the returned value.
}
\description{
Note: `corrgram()` uses base R graphics.
The corrgram function produces a graphical display of a correlation matrix,
called a correlogram.  The cells of the matrix can be shaded or colored to
show the correlation value. Automatic variable reordering can be used to
improve the visualization.
}
\details{
Note: Use the 'col.regions' argument to specify colors.

Non-numeric columns in the data will be ignored.

The off-diagonal panels are specified with \code{panel.pts},
\code{panel.pie}, \code{panel.shade}, \code{panel.fill}, \code{panel.bar},
\code{panel.ellipse}, \code{panel.conf}. \code{panel.cor}.

Diagonal panels are specified with \code{panel.txt}, \code{panel.minmax},
\code{panel.density}.

Use a NULL panel to omit drawing the panel.

This function is basically a modification of the \code{pairs.default}
function with the use of customized panel functions.

The panel.conf function uses \code{cor.test} and calculates pearson
correlations.  Confidence intervals are not available in \code{cor.test} for
other methods (kendall, spearman).

You can create your own panel functions by starting with one of the included
panel functions and making suitable modifications.  Note that because of the
way the panel functions are called inside the main function, your custom
panel function must include the arguments shown in the \code{panel.pts}
function, even if the custom panel function does not use those arguments!
}
\examples{

# To reproduce the figures in Michael Friendly's paper, see the
# vignette, or see the file 'friendly.r' in this package's
# test directory.

# Demonstrate density panel, correlation confidence panel
corrgram(iris, lower.panel=panel.pts, upper.panel=panel.conf,
         diag.panel=panel.density)

# Demonstrate panel.shade, panel.pie, principal component ordering
vars2 <- c("Assists","Atbat","Errors","Hits","Homer","logSal",
           "Putouts","RBI","Runs","Walks","Years")
corrgram(baseball[vars2], order=TRUE, main="Baseball data PC2/PC1 order",
         lower.panel=panel.shade, upper.panel=panel.pie)

# Use reordering options from package seriation (only available if
#    package seriation is installed)
if ("seriation" \%in\% rownames(installed.packages())) {
corrgram(baseball[vars2], order= "OLO",
          main="Baseball data with optimal leaf order",
          lower.panel=panel.shade, upper.panel=panel.pie)

corrgram(baseball[vars2], order= "R2E",
          main="Baseball data with rank 2 ellipse seriation",
          lower.panel=panel.shade, upper.panel=panel.pie)

# the following seriation methods are available
seriation::list_seriation_methods("dist")
}

# CAUTION: The latticeExtra package also has a 'panel.ellipse' function
# that clashes with the same-named function in corrgram. In order to use
# the right one, the example below uses 'lower.panel=corrgram::panel.ellipse'.
# If you do not have latticeExtra loaded, you can just use
# 'lower.panel=panel.ellipse'.

# Demonstrate panel.bar, panel.ellipse, panel.minmax, col.regions
corrgram(auto, order=TRUE, main="Auto data (PC order)",
         lower.panel=corrgram::panel.ellipse,
         upper.panel=panel.bar, diag.panel=panel.minmax,
         col.regions=colorRampPalette(c("darkgoldenrod4", "burlywood1",
                                        "darkkhaki", "darkgreen")))

# 'vote' is a correlation matrix, not a data frame
corrgram(vote, order=TRUE, upper.panel=panel.cor)


# outer labels, all options, larger margins, xlab, ylab
labs=colnames(state.x77)
corrgram(state.x77, oma=c(7, 7, 2, 2),
         outer.labels=list(bottom=list(labels=labs,cex=1.5,srt=60),
                           left=list(labels=labs,cex=1.5,srt=30,adj=c(1,0))))
mtext("Bottom", side=1, cex=2, line = -1.5, outer=TRUE, xpd=NA)
mtext("Left", side=2, cex=2, line = -1.5, outer=TRUE, xpd=NA)

}
\references{
Friendly, Michael.  2002.  Corrgrams: Exploratory Displays for
Correlation Matrices.  \emph{The American Statistician}, 56, 316--324.
\url{http://datavis.ca/papers/corrgram.pdf}

D. J. Murdoch and E. D. Chow. 1996. A Graphical Display of Large Correlation
Matrices. The American Statistician, 50, 178-180.

Hahsler M, Hornik K, Buchta C. 2008. Getting things in order: An
introduction to the R package seriation.
\emph{Journal of Statistical Software}, 25(3), 1--34.
ISSN 1548-7660, \doi{10.18637/jss.v025.i03}
}
\author{
Kevin Wright
}
\keyword{hplot}
